from __future__ import with_statement
import os
import shutil
import syck
import node
import vcs
import backend
from user import UserObject
from lockers.request import LockersRequest

from webob.exc import HTTPFound
from webob import Response

# Assumes we're located in the lockers subdir of LOCKERSROOT
# Needs to be updated if we ever do an actual install
LOCKERSROOT = os.path.dirname(os.path.dirname(__file__))
CONFIGFILE = os.path.join(LOCKERSROOT, 'Config.yaml')
CONFIG = syck.load(file(CONFIGFILE))
REPOROOT = os.path.join(LOCKERSROOT, os.path.expanduser(CONFIG['repo']))
TEMPLATEROOT = os.path.join(LOCKERSROOT, 'templates')
TEMPLATEROOTS = [
        os.path.join(REPOROOT, 'templates'),
        os.path.join(TEMPLATEROOT),
        ]
try:
    os.makedirs(REPOROOT)
except: # who cares
    pass

import template

class LockersApplication(object):
    '''Base class for modules.

    This is also the class responsible for initializing a new
    repository. This kinda sucks because by the time we get here,
    we're already handling a request! This really needs to be fixed.
    '''
    def __init__(self):
        os.chdir(REPOROOT)
        self.vcs = getattr(vcs, CONFIG['type'].capitalize())()
        self.vcs.create()
        if self.vcs.is_new():
            # FIXME: seems to commit multiple things into one commit?
            shutil.copytree(TEMPLATEROOT, 'templates')
            self.vcs.add(['templates'])
            self.vcs.commit(['templates'],'Initial commit, including templates')

        self.backend = backend.make_backend(self.vcs)

    def __call__(self, request):
        request.node = node.Node(request)
        with self.backend.new_transaction() as t:
            user = UserObject.get(t, request)
            request.user = user

    def render_to_response(self, request, filename, ctxt):
        ctxt.setdefault('user', request.user)
        ctxt.setdefault('node', request.node)
        return template.render_to_response(filename, ctxt)

    def need_login(self, request, reason):
        return need_login(request, reason)

def need_login(request, reason):
    # FIXME: request.flash
    return HTTPFound(reason, location=request.node.url_to(module='user', path='login', reason=reason))

def needs_auth(func):
    def auth_wrapper(*args, **kwargs):
        if isinstance(args[0], LockersRequest):
            request = args[0]
        elif isinstance(args[1], LockersRequest):
            request = args[1]
        else:
            raise TypeError, 'request must be first argument to %s'%(func.func_name,)
        if not request.user.is_authenticated():
            # FIXME: optional message?
            return need_login(request, 'You need to be logged in to access this page.')
        return func(*args, **kwargs)

    return auth_wrapper

def complain_if_revision(func):
    '''Decorator to ensure that "edit" requests don't get called with revision arguments.'''
    def revision_wrapper(*args, **kwargs):
        if isinstance(args[0], LockersRequest):
            request = args[0]
        elif isinstance(args[1], LockersRequest):
            request = args[1]
        else:
            raise TypeError, 'request must be first argument to %s'%(func.func_name,)
        if request.node.revision:
            raise ValueError, 'cannot access this method with a revision argument'
        return func(*args, **kwargs)

    return revision_wrapper
