import os
import tarfile
import zipfile
# Some code to handle archived files

class Archive(object):
    def __init__(self, filename, mode):
        super(Archive, self).__init__()
        self.filename = filename
        self.mode = mode

    def __repr__(self):
        return '<%s file [%s] at %s>'%(self.__class__.__name__, self.filename, id(self))

    def listfiles(self):
        raise NotImplementedError, 'cannot list from %s'%self

    def is_directory(self, fname):
        raise NotImplementedError, 'cannot check if file is directory in %s'%self

    def extract_as(self, filename, name):
        raise NotImplementedError, 'cannot extract from %s'%self

class ZipFile(Archive):
    def __init__(self, filename, mode):
        super(ZipFile, self).__init__(filename, mode)
        self.z = zipfile.ZipFile(filename, mode)

    def listfiles(self):
        return self.z.namelist()

    def is_directory(self, fname):
        return fname.endswith('/')

    def extract_as(self, filename, name):
        if filename[-1] == '/':
            # I can't believe the zipfile module doesn't handle this
            raise IOError, "cannot extract directories"
        s = self.z.read(filename)
        f = file(name, 'w')
        f.write(s)
        f.close()

    def write(self, filename, contents):
        self.z.writestr(filename, contents)

class RarFile(Archive):
    pass

class GZipFile(Archive):
    pass

class TarFile(Archive):
    pass

mimetype_to_class = {
    'application/zip': ZipFile,
    'application/x-rar': RarFile,
    'application/x-gzip': GZipFile
}

ext_to_class = {
    'zip': ZipFile,
    'rar': RarFile,
    'tgz': GZipFile,
    'gz': GZipFile,
    'tar': TarFile
}

def can_handle(content_type, filename):
    # FIXME: be smarter about .tar.gz, .tgz
    if content_type in mimetype_to_class:
        return mimetype_to_class[content_type]

    # If we got an application/octet-stream, this means somebody didn't know
    # what type it really was
    if content_type != 'application/octet-stream':
        return None

    name, ext = os.path.splitext(filename)
    ext = ext[1:] # .zip -> zip
    return ext_to_class.get(ext)

def open(content_type, filename, path):
    '''Open an archive file.

    The filename argument is the name the file may have once had; path is where
    it lives now (may not have an extension at all, for example).
    '''
    return can_handle(content_type, filename)(path, 'r')
