import syck.dumpers
import _syck
from models import fields

class LockersDumper(syck.dumpers.Dumper):
    '''Specialized dumper that generates "clean" YAML.

    I don't care whether my object comes out as Unicode or str. They're pretty
    interchangable anyhow. So get this !python/str and !python/unicode out of
    my files. Maybe this is misguided and maybe I shouldn't care about how my
    strings serialize, but for some reason it bothers me.

    The YAML loader is "smart" in that if it sees a string which looks like
    valid ASCII text, it returns a str object, but if it contains nonascii it's
    treated as Unicode and a unicode object is returned. So we just serialize
    to YAML "str" objects and figure whatever we get will be fine.

    Additionally, any object that doesn't want to be serialized "normally" can
    provide a __for_yaml__ method, which will be called to find an object
    to serialize instead of the object itself.
    '''
    def represent_unicode(self, object):
        try:
            return _syck.Scalar(object.encode('ascii'), tag="tag:yaml.org,2002:str")
        except UnicodeEncodeError:
            return _syck.Scalar(object.encode('utf-8'), tag="tag:yaml.org,2002:str")

    def represent(self, object):
        if hasattr(object, '__for_yaml__'):
            return super(LockersDumper, self).represent(object.__for_yaml__())

        return super(LockersDumper, self).represent(object)

    def find_representer(self, object):
        if isinstance(object, fields.Part):
            return self.represent_part

        return super(LockersDumper, self).find_representer(object)

    def represent_part(self, object):
        return self.represent_dict(object.__dict__)

    def represent_str(self, object):
        try:
            return _syck.Scalar(object.encode('ascii'), tag="tag:yaml.org,2002:str")
        except UnicodeDecodeError:
            try:
                return _syck.Scalar(unicode(object, 'utf-8').encode('utf-8'),
                        tag="tag:yaml.org,2002:str")
            except UnicodeDecodeError:
                return _syck.Scalar(object.encode('base64'),
                        tag="tag:yaml.org,2002:binary")

    def represent_dict(self, object):
        items = object.items()
        items.sort()
        return _syck.Map(items, tag="tag:yaml.org,2002:map")

def yaml_dump(o):
    return syck.dump(o, Dumper=LockersDumper)

