#! /usr/bin/python
# Format class: for the formatting of text.
# Currently, Lockers is expected to have a formatting language based on RST,
# with some minor additions such as [[wikilinks]] and [[:commands]].
# Also, since RST (and many other plaintext formatting languages) involve
# using whitespace to delimit blocks, perhaps some kind of {{}} directive
# which would indent and reflow text.
# Ideally, Lockers would support numerous formatting languages. Perhaps these
# would all inherit from this Format class. Shrug?

# This is why ikiwiki changed to a preprocessing behavior rather than a postprocessing one:
# http://git.ikiwiki.info/?p=ikiwiki;a=commit;h=7a5ae22e5a5789bf2b1be432cdc7551ab78a5241

from django.utils import safestring # contrib

class Format(object):
    def __init__(self):
        super(Format, self).__init__(self)

    def format(self, str):
        str = self.preprocess(str)
        str = self._format(str)
        str = self.postprocess(str)
        return safestring.mark_safe(str)

    def preprocess(self, str):
        return str

    def _format(self, str):
        return str

    def postprocess(self, str):
        return str

class FormattedString(object):
    '''String wrapper which "promises" to format to RST.

    This is useful for Django templates -- we can wrap a string with
    this, and when it gets rendered in a Django template, it gets
    coerced to formatted HTML. If you want to use it in a form,
    though, you probably want to call FormattedString.raw() to get the
    "real" data.

    This is how the magic of TextField happens.'''
    def __init__(self, s, format):
        if isinstance(s, FormattedString):
            raise ValueError, 'this is almost certainly a mistake'
        self.contents = s
        self.format = format()

    def __for_yaml__(self):
        return self.contents

    def __unicode__(self):
        formatted = self.format.format(self.contents)
        return safestring.mark_safe(formatted)

    def raw(self):
        return self.contents

    def __eq__(self, rhs):
        if isinstance(rhs, (str, unicode)):
            return self.contents == rhs
        return self.contents == rhs.contents
