#! /usr/bin/python
import re
from docutils import core
from docutils import io
from docutils.parsers.rst import roles
from docutils import nodes
from docutils import utils
from lockers import format
from lockers import backend
from lockers import node

# Getting the right parts of the RST document:
# http://docutils.sourceforge.net/FAQ.html#how-can-i-retrieve-the-body-of-the-html-document
# http://svn.w4py.org/Wiki/trunk/lib/convert_rest.py
#   they do:
#      def _clean_html(html):
#           return html[html.find('<body>')+6:html.find('</body>')]

# If you want to look through other wikis, check:
# http://docutils.sourceforge.net/FAQ.html#are-there-any-wikis-that-use-restructuredtext-syntax

# Bear these in mind:
# http://docutils.sourceforge.net/docs/howto/security.html

# Currently the plan for supporting syntax elaborations is to convert them in
# preprocessing into RST directives:
# [[memes/Leroy Jenkins]]
# becomes:
# |Leroy Jenkins|
# .. |Leroy Jenkins| wikilink:: memes/Leroy Jenkins
# with the heavy lifting done in RST directives. Should this be done using
# a docutils Reader? Don't think so.
# This should get us around the troubles ikiwiki had with markdown.
# Alternately, check Interpreted Text Roles, they look much like what we want.
# http://docutils.sourceforge.net/docs/howto/rst-directives.html
# http://docutils.sourceforge.net/docs/howto/rst-roles.html

def wikilink_role(name, rawtext, text, lineno, inliner,
                  options={}, content=[]):
    current = inliner.document.settings.ctx
    roles.set_classes(options)
    match = inliner.document.settings.db.best_link(current, text)
    if not match:
        node1 = nodes.reference(rawtext, utils.unescape("?"), refuri=text,
                               **options)
        node1["classes"].append("wikilink")
        node2 = nodes.Text(text)
        return [node1, node2], []
    else:
        node = nodes.reference(rawtext, utils.unescape(text), refuri=text,
                               **options)
        node["classes"].append("wikilink")
        return [node], []

# MAINTAIN THESE FUNCTIONS WITH THE ABOVE
INVALID_LINK_RE = re.compile(r'<a ([^>]*?)>\?</a>')
VALID_LINK_RE =   re.compile(r'<a ([^>]*?)>([\w\?]+)</a>')
HREF_RE = re.compile(r'href="([^"]*?)"')
def test_invalid_link(s):
    # Checks if a string looks like it could have been made from an invalid link
    m = INVALID_LINK_RE.search(s)
    if not m: return m
    return HREF_RE.search(m.groups()[0]).groups()[0]

def test_valid_link(s):
    m = VALID_LINK_RE.search(s)
    if not m: return m
    return HREF_RE.search(m.groups()[0]).groups()[0]


def rst_error():
    # Keeping this handy because I still don't understand it.
    msg = inliner.reporter.error(
        'Wikilink to invalid page: %s does not exist.' % rawtext, line=lineno)
    prb = inliner.problematic(rawtext, rawtext, msg)
    return [prb], [msg]

roles.register_local_role('wikilink', wikilink_role)

class RST(format.Format):
    def preprocess(self, s):
        # FIXME: [^\]]*
        # FIXME: escape link name in case it has any `
        wlre = re.compile('\[\[(.*)\]\]')
        while True:
            m = wlre.search(s)
            if not m: break
            pref = s[:m.start()]
            suf = s[m.end():]
            lname = m.groups()[0]
            lname = lname.replace('`', r'\`')   # FIXME: generates \x00 output?
            # This is the interpreted text role:
            s = pref + ':wikilink:`%s`'%lname + suf
            # This is with the directive:
            #s = pref + '|' + lname + '|' + suf + '\n\n.. |%s| wikilink::'%lname

        return s


    def _format(self, str):
        dct = core.publish_parts(source=str, writer_name='html',
                                 settings_overrides={
            'db': backend.get_backend(),
            'ctx': node.Node('wiki', '/'),
            }
                                 )
        return dct['body']

