from lockers.node import Node
from lockers.models.models import UntrackedObject, ModelFactory
from lockers.models import fields

class Indexes(object):
    def __init__(self, t):
        self.transaction = t

    def register_dependencies(self, node, dependencies):
        if not isinstance(dependencies, (tuple, list)):
            dependencies = [dependencies]

        d_obj = CacheDependency.get(self.transaction)
        d_obj.add_dependencies(node, dependencies)
        d_obj.save()

        return self

    def was_changed(self, node):
        # FIXME: factor out as a call_hook or something?
        dependencies = CacheDependency.get(self.transaction)
        dependencies.was_changed(node)
        dependencies.save()

        return self

# Backend hook classes
#
# These classes
class CacheDependency_1(UntrackedObject):
    '''An object that keeps track of the dependencies used to compute untracked objects

    Many untracked objects are cached computations of tracked
    objects. This class provides support for tracking the dependencies
    of the cached objects and deleting them when their dependencies
    change.
    '''
    # If self.forwards[x] = [a, b], this means x depends on a and b.
    forwards = fields.DictField()
    # If self.backwards[x] = [a, b], this means a and b depend on x.
    backwards = fields.DictField()

    def add_dependencies(self, node, dependencies):
        assert node.untracked
        if node not in self.forwards:
            self.forwards[node] = []
        self.forwards[node].extend(dependencies)

        for dependency in dependencies:
            if dependency not in self.backwards:
                self.backwards[dependency] = []
            self.backwards[dependency].append(node)

    def was_changed(self, node):
        forwards_deleted = {}
        if node in self.backwards:
            # Delete self.backwards[node] so that we don't try to delete
            # the node again!
            for target in self.backwards.pop(node):
                # Note: could have already been deleted in this function
                if target not in forwards_deleted:
                    forwards_deleted[target] = True
                    self.get_transaction().remove(target)
                    # FIXME: maybe some kind of notification: "hey, every hook,
                    # we deleted this file!"
                    del self.forwards[target]
                else:
                    print 'already deleted', target

class CacheFactory(ModelFactory):
    basename = 'CacheDependency'
    location = Node('cache', 'dependencies/untracked', untracked=True)

    def get(self, t):
        return super(CacheFactory, self).get(t, self.location)

CacheDependency = CacheFactory()


