'''This is all very sketchy'''

import os
from lockers.node import Node
from lockers.vcs import VCSException

class Cursor(object):
    def __init__(self, query):
        self.generator = query.generate()

    def get_one(self):
        return self.generator.next()

class Query(object):
    '''All queries are rooted at a TransactionQuery'''
    def __init__(self, parent=None):
        self.parent = parent

    def execute(self):
        return Cursor(self)

    def next_child(self, path, revision=None):
        return NextChildQuery(self, path, revision)

    def path(self, path):
        return PathQuery(self, path)

    def node(self, node):
        return NodeQuery(self, node)

    def children(self, *args, **kwargs):
        return ChildrenQuery(self, *args, **kwargs)

    def sort(self, criteria):
        return SortedQuery(self, criteria)

    def generate(self):
        if parent:
            return self.parent.generate()
        raise ArgumentError, 'cannot generate results for empty query'

    def find_transaction(self):
        q = self
        while q and not isinstance(q, TransactionQuery):
            q = q.parent
        return q.t

    def sort(self, **args):
        return QuerySorter(self, args)

    def limit(self, n):
        return QueryLimiter(self, n)

class TransactionQuery(Query):
    def __init__(self, t):
        super(TransactionQuery, self).__init__()
        self.t = t

class PathQuery(Query):
    def __init__(self, parent, path):
        super(PathQuery, self).__init__(parent)
        self.path = path

    def generate(self):
        yield self.path

class NodeQuery(Query):
    def __init__(self, parent, node):
        if not isinstance(node, Node): raise TypeError, '1st argument must be a Node'
        super(NodeQuery, self).__init__(parent)
        self.node = node

    def generate(self):
        yield self.node

class Filter(object):
    def __init__(self, object=None, path_matches=None, path_not_matches=None,
                 isinstance=None):
        self.criteria = []
        if object != None or isinstance != None:
            self.criteria.append(ObjectFilter(object=object,
                                              isinstance=isinstance))
        if path_matches != None or path_not_matches != None:
            self.criteria.append(PathMatchFilter(path_matches))
            self.criteria.append(PathMatchFilter(path_not_matches, invert=True))

    def check(self, t, thing):
        for c in self.criteria:
            if not c.check(t, thing):
                return False

        return True

class ObjectFilter(Filter):
    def __init__(self, object=None, isinstance=None):
        if object == None: object = True
        self.object = object
        self.isinstance = isinstance

    def check(self, t, thing):
        if self.object and not t.exists(thing.filename()):
            return False
        if not self.object and t.exists(thing.filename()):
            return False

        # FIXME: isinstance
        return True

class PathMatchFilter(Filter):
    def __init__(self, expression, invert=False):
        self.expression = expression
        self.invert = invert

    def check(self, t, thing):
        # FIXME: any kind of checking at all
        return True

class ChildrenQuery(Query):
    def __init__(self, parent, *args, **kwargs):
        super(ChildrenQuery, self).__init__(parent)
        self.filter = Filter(**kwargs)

    def generate(self):
        t = self.find_transaction()
        for path in self.parent.generate():
            revision = None
            if isinstance(path, Node): # FIXME: deprecate non-node form?
                revision = path.revision
                path = path.location()
            if not t.exists(path, revision): continue

            for child in t.listdir(path, revision):
                nmodule = path[:path.index('/')]
                npath = os.path.join(path, child)
                npath = npath[npath.index('/'):]
                n = Node(nmodule, npath, revision=revision)
                if self.filter.check(t, n):
                    yield n


class NextChildQuery(Query):
    '''assumes path contains nothing but child directories named numerically'''
    def __init__(self, parent, path, revision):
        '''node is a "bound node" -- node + t?'''
        super(NextChildQuery, self).__init__(parent)
        self.path = path
        self.revision = revision

    def generate(self):
        t = self.find_transaction()
        try:
            dir = t.listdir(self.path, self.revision)
        except VCSException, e:
            # FIXME: hopefully it was a file-not-found
            return
        max = dir[0]
        for i in range(1, len(dir)):
            if max < dir[i]: max = dir[i]

        next = int(max)+1
        fmt = "%%0.%dd"%len(max)  # "%0.9d"

        yield os.path.join(self.path, fmt%next)


class QuerySorter(Query):
    '''Querying is hard; let's go shopping'''
    def __init__(self, parent, criteria):
        super(QuerySorter, self).__init__(parent)
        self.criteria = criteria

    def generate(self):
        l = [n for n in self.parent.generate()]
        l.sort(**self.criteria)
        for i in l: yield i

class QueryLimiter(Query):
    def __init__(self, parent, n):
        super(QueryLimiter, self).__init__(parent)
        self.number = n

    def generate(self):
        g = self.parent.generate()
        for i in range(self.number):
            yield g.next()
