import urllib
from webob import environ_getter, MultiDict
from django.core.handlers import wsgi
from django.utils import datastructures
#datastructures.MultiValueDict = MultiDict

def getone(self, key):
    v = self.getlist(key)
    if not v:
        raise KeyError('Key not found: %r' % key)
    if len(v) > 1:
        raise KeyError('Multiple values match %r: %r' % (key, v))
    return v[0]

datastructures.MultiValueDict.getone = getone

class LockersRequest(wsgi.WSGIRequest):
    # Merge the django WSGIRequest with some handy methods from Webob
    # Multiple inheritance failed big
    # Code is stolen wholeheartedly from webob
    path_info = environ_getter('PATH_INFO')
    script_name = environ_getter('SCRIPT_NAME')
    referrer = environ_getter('HTTP_REFERER')
    def path_info_pop(self):
        """
        'Pops' off the next segment of PATH_INFO, pushing it onto
        SCRIPT_NAME, and returning the popped segment.  Returns None if
        there is nothing left on PATH_INFO.

        Does not return ``''`` when there's an empty segment (like
        ``/path//path``); these segments are just ignored.
        """
        path = self.path_info
        if not path:
            return None
        while path.startswith('/'):
            self.script_name += '/'
            path = path[1:]
        if '/' not in path:
            self.script_name += path
            self.path_info = ''
            return path
        else:
            segment, path = path.split('/', 1)
            self.path_info = '/' + path
            self.script_name += segment
            return segment

    def path_info_peek(self):
        """
        Returns the next segment on PATH_INFO, or None if there is no
        next segment.  Doesn't modify the environment.
        """
        path = self.path_info
        if not path:
            return None
        path = path.lstrip('/')
        return path.split('/', 1)[0]

    def application_url(self):
        """
        The URL including SCRIPT_NAME (no PATH_INFO or query string)
        """
        return self.host_url + urllib.quote(self.environ.get('SCRIPT_NAME', ''))
    application_url = property(application_url, doc=application_url.__doc__)

    def host_url(self):
        """
        The URL through the host (no path)
        """
        e = self.environ
        url = e['wsgi.url_scheme'] + '://'
        if e.get('HTTP_HOST'):
            host = e['HTTP_HOST']
            if ':' in host:
                host, port = host.split(':', 1)
            else:

                port = None
        else:
            host = e['SERVER_NAME']
            port = e['SERVER_PORT']
        if self.environ['wsgi.url_scheme'] == 'https':
            if port == '443':
                port = None
        elif self.environ['wsgi.url_scheme'] == 'http':
            if port == '80':
                port = None
        url += host
        if port:
            url += ':%s' % port
        return url
    host_url = property(host_url, doc=host_url.__doc__)

    @classmethod
    def blank(cls, uri, post={}):
        query = ''
        if '?' in uri:
            query = uri[uri.find('?')+1:]
            uri = uri[:uri.find('?')]
        o = cls({
                'wsgi.multiprocess': False,
                'wsgi.version': (1, 0),
                'SERVER_NAME': 'localhost',
                'wsgi.run_once': False,
                'wsgi.multithread': False,
                'SCRIPT_NAME': '',
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET',
                'HTTP_HOST': 'localhost:80',
                'PATH_INFO': uri,
                'SERVER_PORT': '80',
                'SERVER_PROTOCOL': 'HTTP/1.0',
                'QUERY_STRING': query})
        o.POST = o.POST.copy()
        o.POST.update(post)
        return o

    # More code stolen from webob.Request
    def call_application(self, application, catch_exc_info=False):
        """
        Call the given WSGI application, returning ``(status_string,
        headerlist, app_iter)``

        Be sure to call ``app_iter.close()`` if it's there.

        If catch_exc_info is true, then returns ``(status_string,
        headerlist, app_iter, exc_info)``, where the fourth item may
        be None, but won't be if there was an exception.  If you don't
        do this and there was an exception, the exception will be
        raised directly.
        """
        captured = []
        output = []
        def start_response(status, headers, exc_info=None):
            if exc_info is not None and not catch_exc_info:
                raise exc_info[0], exc_info[1], exc_info[2]
            captured[:] = [status, headers, exc_info]
            return output.append
        app_iter = application(self.environ, start_response)
        if (not captured
            or output):
            try:
                output.extend(app_iter)
            finally:
                if hasattr(app_iter, 'close'):
                    app_iter.close()
            app_iter = output
        if catch_exc_info:
            return (captured[0], captured[1], app_iter, captured[2])
        else:
            return (captured[0], captured[1], app_iter)




import webob
#LockersRequest = webob.Request
