import os.path
from webob import Response

from django import template
from django.template import Template
import django.template.loader
Context = template.Context
from application import TEMPLATEROOTS

# FIXME: This sucks. Wish there was a memoize decorator included in Python 2.5

__templateCache = {}

def load_template(fname):
    # Memoize, cribbed from the Python Decorator Library
    # FIXME: this is totally wrong
    try:
        return __templateCache[fname]
    except KeyError:
        for dir in TEMPLATEROOTS:
            tryfile = os.path.join(dir, fname)
            if os.path.exists(tryfile):
                contents = file(tryfile).read()
                # FIXME: "should" be fname, but disabling this
                # memoization is fine for now
                __templateCache[tryfile] = value = template.Template(contents)
                return value
        raise IOError, "Template could not be found: %s in %s"%(fname, TEMPLATEROOTS)

def render_template(fname, context):
    return load_template(fname).render(template.Context(context))

def render_to_response(tname, context):
    t = load_template(tname)
    c = template.Context(context)
    return Response(t.render(c))

def url_to_tag(parser, token):
    '''The url_to tag is very very clumsy so far. It relies on there being a
    'node' object in the context, but at least the syntax is a little flexible.

    Example usage:
    {% url_to revision=version, action = "edit" %}
    {% url_to object.get_node, action="edit"
    '''
    text = ' '.join(token.contents.split()[1:])
    if ',' not in text and '=' not in text:
        return Url_to_TagNode_Expr(text)
    pairs = text.split(',')
    nodename = ''
    vararg = {}
    litarg = {}
    first = True
    for pair in pairs:
        if '=' not in pair and first:
            nodename = pair.strip()
            first = False
            continue
        first = False
        key, val = pair.split('=')
        key = key.strip()
        val = val.strip()
        if (val[0] == '"' and val[-1] == '"') \
                or (val[0] == "'" and val[-1] == "'"):
            val = val[1:-1]
            litarg[str(key)] = str(val)
        else:
            vararg[str(key)] = str(val)

    return Url_to_TagNode(nodename, vararg, litarg)

# FIXME: use the template.Variable class when I get a new Django version
class Url_to_TagNode(template.Node):
    def __init__(self, nodename, vararg, litarg):
        self.nodename = nodename
        self.vararg = vararg
        self.litarg = litarg

    def render(self, context):
        target = None
        if self.nodename:
            target = template.resolve_variable(self.nodename, context)

        kwargs = dict(self.litarg)
        for key, value in self.vararg.items():
            kwargs[key] = template.resolve_variable(value, context)

        node = context.get('node')
        return template.escape(node.url_to(target, **kwargs))

class Url_to_TagNode_Expr(template.Node):
    def __init__(self, node):
        super(Url_to_TagNode_Expr, self).__init__()
        self.node = node

    def render(self, context):
        node = template.resolve_variable(self.node, context)
        return context.get('node').url_to(node)

template.defaulttags.register.tag('url_to', url_to_tag)

class BreadcrumbsTagNode(template.Node):
    def __init__(self):
        super(BreadcrumbsTagNode, self).__init__()

    def render(self, context):
        node = context.get('node')
        crumbs = []
        crumbs.append({'url': node.url_to(module=''), 'name': 'Lockers'})
        crumbs.append({'url': node.url_to(path=''), 'name': node.module.capitalize()})

        # FIXME: multiple slashes
        path = node.path.strip('/').split('/')
        sumpath = ''
        join = os.path.join
        for pathpart in path:
            if not pathpart: continue
            sumpath = join(sumpath, pathpart)
            crumbs.append({'url': node.url_to(path=sumpath), 'name': pathpart})

        return load_template('breadcrumbs.html').render(template.Context({
                'node': node,
                'crumbs': crumbs[:-1],
                'lastcrumb': crumbs[-1]
                }))

def breadcrumbs(parser, token):
    '''Also relies on there being a 'node' object in the context.'''
    return BreadcrumbsTagNode()

template.defaulttags.register.tag('breadcrumbs', breadcrumbs)

@template.defaulttags.register.tag
def asset(parser, token):
    '''Still need that 'node' object.'''
    # Supports multiple names, separated by either spaces or commas.
    # First, split on spaces, then rejoin using commas (after
    # stripping 'asset'). Then split on commas to get all the names.
    text = ','.join(token.contents.split()[1:])
    names = text.split(',')

    return Asset_TagNode(names)

ASSET_FORMATS = {
    '.js': '<script src="%s" type="text/javascript"></script>',
    '.css': '<link rel="stylesheet" href="%s" />',
    '.png': '<img src="%s" alt="%s" />',
    '.gif': '<img src="%s" alt="%s" />',
    }
class Asset_TagNode(template.Node):
    def __init__(self, names):
        super(Asset_TagNode, self).__init__(self)
        self.names = names

    def render(self, context):
        node = context.get('node')
        # FIXME: check for config; allow putting static assets somewhere else,
        # CDN, etc.
        newtags = []

        for name in self.names:
            if name[0] == name[-1] and (name[0] in ('"', "'")):
                name = name[1:-1]
            else:
                name = template.resolve_variable(name, context)
            fmt = ASSET_FORMATS[os.path.splitext(name)[1]]
            src = node.url_to(module='assets', path=name)
            filename = os.path.basename(name)
            args = (src, filename)
            fmtnum = fmt.count('%')
            newtags.append(fmt%args[:fmtnum])

        return ''.join(newtags)

def htmlid(s):
    return s.replace('/', '-')

template.defaultfilters.register.filter('htmlid', htmlid)
