import os
import sys
import select
import tempfile

def du(dir):
    p = os.popen('du -s %s'% dir)
    s = p.read()
    return int(s.split('\t')[0])

def forkl(path, *args):
    pid = os.fork()
    if pid == 0:
        os.execl(path, *args)
    else:
        somepid, stat = os.waitpid(pid, 0)
        return stat

def pexecl(path, *args):
    read, write = os.pipe()
    readerr, writeerr = os.pipe()
    pid = os.fork()
    if pid == 0:
        os.close(read)
        os.close(readerr)
#        os.close(sys.stdin.fileno())   # darcs hates closed stdin
        os.dup2(write, 1)    # sys.stdout.fileno() # testing messes this up
        os.dup2(writeerr, 2)
        os.execl(path, *args)
    else:
        os.close(write)                 # Be sure to do this
        os.close(writeerr)
        output = {readerr: '', read: ''}
        openfds = [read, readerr]

        while True:
            readfds, writefds, errfds = select.select(openfds, [], openfds)
            for fd in readfds:
                s = os.read(fd, 1024)
                if s == '': openfds.remove(fd)
                else: output[fd] += s
            if not openfds: break

        somepid, stat = os.waitpid(pid, 0)
        os.close(read)
        os.close(readerr)
    return stat, output[read], output[readerr]

class pexecl_stream(object):
    '''Lazy creation of streams to read from commands'''
    def __init__(self, path, *args):
        self.path = path
        self.args = args

    def __iter__(self):
        return pipe_iter(self.path, *self.args)


class pipe_iter(object):
    CHUNK_SIZE= 32*1024
    def __init__(self, path, *args):
        read, write = os.pipe()
        pid = os.fork()
        if pid == 0:
            os.close(read)
        #        os.close(sys.stdin.fileno())   # darcs hates closed stdin
            os.dup2(write, 1)
            os.execl(path, *args)
        else:
            os.close(write)                 # Be sure to do this
            f = os.fdopen(read)
        self.pid = pid
        self.file = f

    def __iter__(self):
        return self

    def next(self):
        s = self.file.read(self.CHUNK_SIZE)
        if s == '':
            self.file.close()
            somepid, stat = os.waitpid(self.pid, 0)
            self.stat = stat
            raise StopIteration

        return s

    def status(self):
        return self.stat


def dump(filename, contents):
    f = file(filename, 'w')
    if contents[-1] != '\n': contents = contents + '\n'
    f.write(contents)
    f.close()

def mimetype(filename):
    # FIXME: mimetypes.guess_type?
    stat, out, err = pexecl('/usr/bin/file', 'file', '--print0', '--mime-type', filename)
    return out.split('\0')[1].strip()

def make_tempfile(signature):
    '''Return the name of a new temporary file.

    Make a temporary file, associated with the lockers component given
    by the signature. (This way we can find out, for example, who's
    leaving all these dead tempfiles lying around.)'''

    try:
        os.makedirs('temp')
    except OSError: pass
    fhandle, name = tempfile.mkstemp(prefix=signature+'-', dir='temp')
    os.close(fhandle)
    return name
