from __future__ import with_statement
import datetime
from lockers import application
import lockers.node
from lockers.format.rst import RST
from webob import Response
import webob.exc
from lockers.vcs import FileNotFoundError, VCSException
from lockers.models import Changeset, LockersObject, DoesNotExistError, ModelFactory, ChangesetFactory, PRESERVE, fields
from lockers.template import render_to_response
from lockers import utils
from contrib.django import newforms
from lockers import archive

import traceback

class FileChangeset_1(Changeset):
    pass

FileChangeset = ChangesetFactory('FileChangeset')

class FileObject_1(LockersObject):
    '''The parts of a FileObject are:

    index: dict of metainfo, including upload time, filename, mimetype,
        description.. what else? Size?
    data: the file itself'''
    # FIXME: deprecated
    COMMENT = FileChangeset
    data = fields.RawPart()
    class index(fields.Part):
        mimetype = fields.StrField()
        uploaded = fields.DateTimeField()
        filename = fields.StrField()
        archive = fields.BoolField()
        description = fields.StrField()

FileObject = ModelFactory('FileObject')

class UploadFileForm(newforms.Form):
    file = newforms.FileField()
    description = newforms.CharField(widget=newforms.Textarea, required=False)
    # As far as I can tell, this field is never used -- maybe delete
    # it and eagerly unpack whatever we can
    unpack = newforms.BooleanField(initial=True)

class FileApplication(application.LockersApplication):
    '''Lockers module for uploading/downloading files.'''
    def __init__(self):
        super(FileApplication, self).__init__()
        self.rst = RST()

    def __call__(self, request):
        super(FileApplication, self).__call__(request)
        node = request.node
        if node.path == 'upload':
            if request.method == 'POST':
                return self.receive(request, node)
            else:
                return self.upload_form(request)

        if ('action', 'send') in request.GET.items():
            return self.send(request, node)
        else:
            return self.view(request, node)

    @classmethod
    def new_file_node(cls, t):
        # FIXME: some kind of query goes here
        try:
            files = t.listdir('files/')
            files.sort()
            lastfile = int(files[-1])
        except VCSException:
            # assume no such directory
            lastfile = 0
        newfile = "%0.9d"%(lastfile+1)

        return lockers.node.Node('files', '/'+newfile)

    @classmethod
    def file_for_request(cls, request, t):
        '''FIXME: Saving *must* be done by caller.

        Needs to be this way because sometimes you don't know if you
        want it to be enclosed until after it's been created,
        unpacked, etc.'''
        file = request.FILES.getone('file')
        meta = {'filename': file.name, 'uploaded': datetime.datetime.now(),
                'mimetype': file.content_type,
                'description': request.POST.getone('description')}

        # FIXME: check if file already exists in repo?
        filenode = cls.new_file_node(t)
        obj = FileObject.new(t, filenode)
        obj.data = file
        obj.index.__dict__.update(meta)
        if archive.can_handle(file.content_type, file.name):
            obj.index.archive = True
        else: obj.index.archive = False
        return obj

    @classmethod
    def unpack(cls, obj, t):
        if not archive.can_handle(obj.index.mimetype, obj.index.filename):
            return None
        filenode = obj.get_node()
        archive_obj = archive.open(obj.index.mimetype, obj.index.filename, filenode.part('data'))
        obj.index.archive_files = []
        archive_nodes = []
        for fname in archive_obj.listfiles():
            if archive_obj.is_directory(fname): continue
            # FIXME: recurse?
            filenode_small = cls.new_file_node(t)
            uobj = FileObject.new(t, filenode_small)
            uobj.index.filename = fname
            uobj.index.uploaded = datetime.datetime.now()
            uobj.index.mimetype = utils.mimetype(filenode_small.part('data'))
            uobj.index.description = 'Automatic unpack: %s from %s'%(
                fname, obj.index.filename)
            uobj.lockersmeta.enclosed_by = filenode.filename()
            uobj.data = ''
            # FIXME: these'll need to be re-enclosed in picture module?
            uobj.save(FileChangeset({'message': 'automatically unpacked file %s from %s'%(fname, obj.index.filename)}),
                      data = lambda filename: \
                          archive_obj.extract_as(fname, filename))
            t.apply() # FIXME
            obj.index.archive_files.append(filenode_small.filename())
            archive_nodes.append(filenode_small)
        return archive_nodes

    def receive(self, request, node):
        #print request, dir(request), request.body
        with self.backend.new_transaction() as t:
            obj = self.file_for_request(request, t)
            obj.save({'message': 'uploaded file %s'%obj.index.filename})

        url = node.url_to(path=obj.get_node().path)

        return webob.exc.HTTPFound(detail='File saved.', location=url)

    def upload_form(self, request):
        return self.render_to_response(request, 'files/upload_form.html', {
                'form': UploadFileForm(),
                })

    def view(self, request, node):
        with self.backend.new_transaction() as t:
            obj = FileObject(t, node)
            if hasattr(obj.index, 'description'):
                description = self.rst.format(obj.index.description)
            else: description = ''
            index = obj.index

        return self.render_to_response(request, 'files/view.html', {
                'filename': index.filename,
                'description': description,
                'uploaded': obj.index.uploaded,
                })

    def send(self, request, node):
        with self.backend.new_transaction() as t:
            obj = FileObject(t, node)
            f = obj.data
            index = obj.index

        r = Response(content_type=index.mimetype)
        r.app_iter = f
        return r

files = FileApplication
