from __future__ import with_statement
import email.utils
import datetime
import hashlib
import traceback
import PIL.Image
import cStringIO
from lockers import application
from lockers.application import needs_auth, complain_if_revision
import lockers.node
import lockers.diff as diff
from lockers import backend
from lockers.format.rst import RST
from webob import Response
from webob.exc import *
import os
from os.path import join
import cgi
import time, datetime
from lockers.vcs import FileNotFoundError, VCSException
from lockers.models.models import Changeset, LockersObject, DoesNotExistError, ModelFactory, UntrackedObject, PRESERVE, AlreadyExistsError, CachedModelFactory, ChangesetFactory
from lockers.models import fields
from lockers import node
from lockers.template import render_to_response, render_template
from contrib.django import newforms
# FIXME: contrib.django, but this references a different SafeString :(
from django.utils.safestring import mark_safe, mark_for_escaping
from lockers import archive, utils, pagination
from contrib import EXIF

import assets

# Currently stuck on how to keep files from being created before we mark
# them as "enclosed".
# 1. Could pass requests to the file module, and specify a "dummy" enclosure.
#    file module would 302 the request back to us afterwards, and we could
#    then work on the data.
# That's pretty hideous, but it's workable.
#
# 2. Could import the file module itself and call methods of the file module,
#    FileObject to get stuff done.  We'd create the file ourselves and tweak
#    it as necessary before saving.
# The problem with this is that we get into trouble with cross-module data.
#
# Changesets should also be enclosed?
# As long as we only add fields, we should be OK. The only problem is that we
# might want to customize the changeset in some way to make it clear
# why the file got uploaded. And if we enclose the changeset too in some way,
# we can then load it differently or something.
import files


# Rotation.
#
# This is going to be tricky, because jhead only works on some camera
# models (and it doesn't seem to include any of the ones I have
# here). So I could: totally refuse to do anything and say it's the
# user's problem, or render thumbs of all the images and let the user
# rotate whichever ones they want, before they become pictures. For
# the now, option #1 might be sufficient, but I'll have to implement
# #2 before I can really call it done. #2 might involve wacky design
# considerations, so I'll at least think about it now.
#
# Actually #2 is kinda silly too. Why not make it two steps, creation
# and rotation? I think that's fine, because that way I don't have to
# worry about "what if someone uploads an image but forgets to rotate
# it and then logs off", or whatever.


# Picture paths are /pictures/<user>/<id>, and album paths are
# /pictures/<user>/albums/<name>.

# Flickr does /pictures/<user>/<id>, but although <id> seems to be
# some kind of GUID, you can't always access a given GUID through an
# arbitrary user. It might make more sense to just use
# /pictures/<id>. Then again maybe not. If we use
# /pictures/<user>/<id>, it'll be much easier to make that backwards
# compatible with /pictures/<id>.. so let's do that. [<id> comes from
# the timestamp of file upload.]

# I also tried /pictures/<user>/pictures/<id> -- but I couldn't bear
# the word "pictures" in the url twice. As long as picture ids remain solely
# numeric it should be OK.

class PictureChangeset_1(Changeset):
    def make_message(self):
        return 'No user-provided message for this change.'

PictureChangeset = ChangesetFactory('PictureChangeset')

class AddPictureChangeset_1(PictureChangeset):
    def get_details(self, backend, request):
        return 'Added a picture.'

AddPictureChangeset = ChangesetFactory('AddPictureChangeset')

class AddAlbumChangeset_1(PictureChangeset):
    def get_details(self, backend, request):
        return [{'icon': 'add.png',
                 'message': 'Creating album.'}]

AddAlbumChangeset = ChangesetFactory('AddAlbumChangeset')

_RST = RST()
class EditPictureChangeset_1(PictureChangeset):
    # This sucks -- how can we make this better?
    caption = fields.DiffField()
    def get_details(self, backend, request):
        lst = []
        if self.caption:
            if self.caption.old:
                message = mark_safe('Changed caption from <div>%s</div> to <div>%s</div>.'%(
                        _RST.format(self.caption.old),
                        _RST.format(self.caption.new)
                        ))
            else:
                message = mark_safe('Set caption to <div>%s</div>.'%(
                        _RST.format(self.caption.new)))

            lst.append({
                    'icon': 'writing.png',
                    'message': message
                    })
        return lst

EditPictureChangeset = ChangesetFactory('EditPictureChangeset')

class EditAlbumChangeset_1(PictureChangeset):
    name = fields.DiffField()
    thumb = fields.DiffField()

    def __init__(self, *args, **kwargs):
        super(EditAlbumChangeset_1, self).__init__(*args, **kwargs)

    def get_details(self, backend, request):
        lst = []
        if self.name:
            lst.append({
                    'icon': 'writing.png',
                    'message': 'Changed title of album from %s to %s.'%(self.name.old, self.name.new)
                    })

        if self.thumb:
            node = request.node
            message = mark_safe('Changed thumbpic: <div class="lockers-picture-compare"><img src="%s" /><div class="lockers-comparison-arrow">-&gt;</div><img src="%s" /></div>'%(node.url_to(self.thumb.old, action="thumb0"), node.url_to(self.thumb.new, action="thumb0")))
            lst.append({
                    'icon': 'make_prettier.png',
                    'message': message
                    })

        return lst

EditAlbumChangeset = ChangesetFactory('EditAlbumChangeset')

class RotationChangeset_1(PictureChangeset):
    # FIXME: NodeField
    album = fields.StrField()
    # FIXME: NodeField
    image = fields.StrField()
    degrees = fields.NumField()
    def make_message(self):
        return 'rotated image via ajax callback'

    def get_details(self, backend, request):
        direction = 'clockwise'
        icon = 'rot_right.png'
        if self.degrees == 270 or self.degrees == -90:
            direction = 'counter-clockwise'
            icon = 'rot_left.png'
        albumname = None
        with backend.new_transaction() as t:
            if self.album:
                # FIXME: existing?
                albumname = AlbumObject.get(t, self.album).title
            filenode = PictureObject(t, self.image).filenode
            filename = files.FileObject(t, filenode, want_fields=['filename']).filename

        if albumname:
            message = 'Rotated %s %s while editing %s' % (filename, direction, albumname)
        else:
            message = 'Rotated %s %s' % (filename, direction,)
        message = mark_for_escaping(message)

        node = request.node
        message += '<div class="lockers-picture-compare"><img src="%s" /><div class="lockers-comparison-arrow">-&gt;</div><img src="%s" /></div>'%(
            node.url_to(self.image, revision=self.last_revision, action='thumb0'),
            node.url_to(self.image, action='thumb0'),
            )
        return [{'message': mark_safe(message),
                 'icon': icon}]

RotationChangeset = ChangesetFactory('RotationChangeset')

class PictureObject_1(LockersObject):
    '''
    filenode: the path to the file this picture represents
    caption: the caption for an image, which may be different from the picture's
      description?
    '''
    filenode = fields.NodeField()
    caption = fields.TextField()
    class exif(fields.Part):
        model = fields.StrField()
        # FIXME: both of these should be like a DateTimeField
        date = fields.Field()
        uploaded = fields.Field()


    def thumbURL(self):
        # FIXME: I'd really like to make a method like this work, but I need the
        # current node so I can do a url_to
        pass

    def get_name(self):
        return self.get_node().last_path_part()

    def get_user(self):
        n = self.get_node()
        p = n.path
        return p[:p.index('/')]

    def read_exifinfo(self, fileobj):
        # We do this on object creation rather than on-the-fly because
        # editing operations could lose the exif data -- for example,
        # jpegtran, unless given a special flag.
        # This makes album creation slower, definitely, but reads are faster.
        if fileobj.is_new():
            tags = EXIF.process_file(cStringIO.StringIO(''.join(fileobj.data)))
            fileobj.data.seek(0)
        else:
            tags = EXIF.process_file(file(fileobj.get_node().part('data')))
        ts = fileobj.uploaded
        if 'Image Model' in tags:
            self.exif.model = tags['Image Model'].values
        else:
            # FIXME: print a warning or something
            self.exif.model = ''
        if 'Image DateTime' in tags:
            self.exif.date = datetime.datetime.strptime(tags['Image DateTime'].values, "%Y:%m:%d %H:%M:%S")
        else: self.exif.date = ''
        self.exif.uploaded = ts


class PictureFactory(ModelFactory):
    basename = 'PictureObject'
    def get_or_dummy(self, t, node, name):
        if node == None:
            return self.get_dummy(t, name)
        return self.get(t, node)

    def get_dummy(self, t, name):
        # FIXME
        return DUMMY_PICTURES[name]

PictureObject = PictureFactory()

class PicturesCache_1(UntrackedObject):
    hash_to_image = fields.DictPart()

PicturesCache = CachedModelFactory('PicturesCache', path='pictures')

class AlbumObject_1(LockersObject):
    # List of Nodes
    pictures = fields.ListField(fields.NodeField())
    title = fields.StrField(default='New Album')
    # FIXME: NodeField
    album_thumb_pic = fields.NodeField()

    def get_id(self):
        return self.get_node().last_path_part()

    def get_image(self):
        if self.album_thumb_pic: return self.album_thumb_pic
        return self.pictures[0]

    def get_zipped_node(self):
        parts = self.get_node().path.split('/')
        userid = parts[0]
        albumid = parts[-1]
        node = self.get_node().replace(module='cache', path=join('pictures', 'zipped', userid, albumid), untracked=True, revision=None)
        if self.get_node().revision:
            node.append(self.get_node().revision)
        return node

AlbumObject = ModelFactory('AlbumObject')

class ZippedAlbum_1(UntrackedObject):
    data = fields.RawPart()
    filename = fields.StrField()

ZippedAlbum = ModelFactory('ZippedAlbum')

# This might not be the best way to handle "No next" and "No previous"
# images. Flickr renders a path to a static image, which is presumably
# CDN'd or something. Originally I intended to render using PIL the
# image on-the-fly but decided that was dumb and just use the assets
# machinery to fetch a given image. This is nice in that I don't have
# to generate an object, plus it will "just work" if you have the
# right image in the repository.
#
# The issues are as follows:
# 1) does the template know that the image is a dummy?
# [My answer: yes, but only a little -- I don't want it to be too hard
# to change the template, so the template doesn't have special-case
# code for generating URLs for dummy images.]
# 2) does the picture module handle the request, or someone else?
# [Right now it's the assets module, but it could change -- just make
# the DUMMY_NODEs point to the assets module.]
# 3) do we serve a static image or do something different?
# [I think static is fine for now. Can't think of too many use cases
# for generating images that read "No next image", plus it would mean
# we have to include a font in the data directory or something.]
DUMMY_NODES = {
    'prev': node.Node(module='assets', path='/pictures/no_previous.png'),
    'next': node.Node(module='assets', path='/pictures/no_next.png')
}

DUMMY_PICTURES = {
    'prev': PictureObject.new(backend.DUMMY_TRANSACTION, DUMMY_NODES['prev'], filenode=DUMMY_NODES['prev']),
    'next': PictureObject.new(backend.DUMMY_TRANSACTION, DUMMY_NODES['next'], filenode=DUMMY_NODES['next'])
}

def hash_image(file):
    h = hashlib.new('sha1')
    if hasattr(file, 'read'):
        h.update(file.read())

        # this works on Django uploaded files, obviating need for temp files
        # (that we didn't create)
        file.seek(0)
    else:
        try:
            for thing in file:
                h.update(thing)
        except TypeError, e:
            # assume string
            h.update(file)
    return h.hexdigest()

class ThumbnailObject_1(UntrackedObject):
    '''
    Thumbnail's basically just a file. FIXME:?
    '''
    data = fields.RawPart()
    class index(files.FileObject_1.index):
        pass

ThumbnailObject = ModelFactory('ThumbnailObject')

class AlbumContainerTracker_1(UntrackedObject):
    album_containing_picture = fields.DictField()

AlbumContainerTracker = ModelFactory('AlbumContainerTracker')

class PictureUploadFileForm(files.UploadFileForm):
    pass

del PictureUploadFileForm.base_fields['unpack']


THUMB_SIZE = [(200, 200), (600, 600)]
SCREEN_SIZE = (1200, 1200)

class ChangesetRenderer(object):
    # Careful! We modify the changesets given to us for rendering.
    # This could be a re-entrancy issue, but we create a new
    # ChangesetRenderer each request, so it should be fine
    def __init__(self, changesets, backend, request):
        self.changesets = changesets
        self.backend = backend
        self.request = request

    def render(self):
        for c in self.changesets:
            if hasattr(c, 'get_details'):
                try:
                    c.details = c.get_details(self.backend, self.request)
                except Exception, e:
                    traceback.print_exc()
            else:
                c.details = []

            for detail in c.details:
                if 'icon' in detail:
                    detail['icon'] = 'icons/'+detail['icon']

        return render_template('changesets.html', {
                'node': self.request.node,
                'changesets': self.changesets,
                })


class PicturesApplication(application.LockersApplication):
    '''Lockers module for uploading/downloading files.'''
    def __init__(self):
        super(PicturesApplication, self).__init__()
        self.rst = RST()

    def __call__(self, request):
        super(PicturesApplication, self).__call__(request)
        node = request.node
        # FIXME: some kind of routes system
        if node.path.startswith('upload'):
            if not request.user.is_authenticated():
                return self.need_login(request, 'You need to be logged in to upload photos.')
            if request.method == 'POST':
                return self.new_file(request)
            return self.upload_form(request)

        if ('action', 'thumb0') in request.GET.items():
            return self.thumb(request, THUMB_SIZE[0], 'size0')

        if ('action', 'thumb1') in request.GET.items():
            return self.thumb(request, THUMB_SIZE[1], 'size1')

        if ('action', 'screen') in request.GET.items():
            return self.thumb(request, SCREEN_SIZE, 'screen')

        if ('action', 'ajax-rotate') in request.GET.items():
            return self.ajax_rotate(request, node)

        pieces = node.path.split('/')
        if len(pieces) == 2 and pieces[1] == 'pictures':
            return self.list_pictures(request, node, pieces[0])

        if len(pieces) >= 2 and pieces[1] == 'albums':
            # Album-specific stuff
            if ('action', 'download-zip') in request.GET.items():
                return self.download_zip(request, node)

            if ('action', 'edit') in request.GET.items():
                return self.edit_album(request, node)

            if ('action', 'history') in request.GET.items():
                return self.history_album(request, node)

            if len(pieces) >= 3:
                return self.display_album(request, node)
            return self.list_albums(request, node, pieces[0])

        # FIXME: routes, good lord
        EXISTS = False
        with self.backend.new_transaction() as t:
            if t.exists(node):
                EXISTS = True
        if EXISTS:
            return self.display_single(request, node)

        if len(pieces) >= 1 and pieces[0]:
            return self.user_gallery(pieces[0], request, node)

        return self.all_gallery(request, node)

    def jhead(self, filename):
        # FIXME: no way to actually determine if jhead did anything, or would do
        # anything.
        pass

    def download_zip(self, request, node):
        '''FIXME: reuse of file-generation and file-sending code'''
        # FIXME: other archive types?

        # FIXME: super arcane
        # FIXME: etag
        with self.backend.new_transaction() as t:
            album = AlbumObject(t, request.node)
            node = album.get_zipped_node()
            if not t.exists(node):
                tempname = utils.make_tempfile('pictures-album-zip')
                z = archive.ZipFile(tempname, 'w')
                filenodes = []
                for f in album.pictures:
                    pic = PictureObject(t, f)
                    filenodes.append(pic.filenode)
                    file = files.FileObject(t, pic.filenode)
                    z.write(file.filename,
                            ''.join(file.data))

                def rename(filename):
                    os.rename(tempname, filename)

                if not t.exists(node):
                    # In case zipping took a long time. This still
                    # doesn't fix the race condition! FIXME.
                    z = ZippedAlbum.new(t, node)
                    z.data = ''
                    z.filename = album.title + '.zip'  # FIXME: encoding
                    z.save(data=rename)
                    z.register_dependencies(filenodes)
                else:
                    os.remove(tempname)

        with self.backend.new_transaction() as t:
            obj = ZippedAlbum(t, node)

        r = Response(content_type='application/zip')
        # r'foo"\ bar' -> r'foo\"\\ bar'
        qfilename = email.utils.quote(obj.filename)
        # -> r'"foo\"\\ bar"'
        r.headers['Content-Disposition'] = 'attachment;filename="%s"'%(qfilename,)
        r.app_iter = obj.data
        return r

    def thumb(self, request, size, cachename):
        with self.backend.new_transaction() as t:
            p = PictureObject.get(t, request.node)
            # FIXME: someplace else to put them?
            # FIXME: untracked objects should not be able to have a revision!
            thumbnode = request.node.replace(module='cache', path=join('pictures', 'thumbnails', cachename, request.node.path), untracked=True, revision=None)
            if request.node.revision: thumbnode.append(request.node.revision)
            if not t.exists(thumbnode):
                file = files.FileObject.get(t, p.filenode)
                # Slurp all the data into memory; this sucks
                data = cStringIO.StringIO(''.join(file.data))
                i = PIL.Image.open(data)
                # FIXME: NEAREST first, then ANTIALIAS later?
                i.thumbnail(size, PIL.Image.ANTIALIAS)
                thumb = ThumbnailObject.new(t, thumbnode)
                thumb.data = ''
                content_type = file.index.mimetype
                thumb.index.mimetype = content_type
                thumb.save(
                    data=lambda filename: i.save(filename, i.format),
                    )
                thumb.register_dependencies([request.node, p.filenode])

        with self.backend.new_transaction() as t:
            thumb = ThumbnailObject.get(t, thumbnode)
        r = Response(content_type=thumb.index.mimetype)
        r.app_iter = thumb.data
        return r

    def new_file(self, request):
        with self.backend.new_transaction() as t:
            obj = files.files.file_for_request(request, t)
            # FIXME: archives?
            # FIXME: jhead


            if obj.index.archive:
                # FIXME: changeset
                obj.save({'message': 'uploaded file %s'%obj.index.filename})
                t.apply()
                contents = files.files.unpack(obj, t)
                albumlist = []
                for afile in contents:
                    try:
                        picnode = self.create_picture(t, request, files.FileObject.existing(t, afile))
                        albumlist.append(picnode)
                    except AlreadyExistsError, e:
                        # FIXME: album links to other picture?
                        pass
                albumid = "%d%06d"%(time.time(), datetime.datetime.now().microsecond)
                albumnode = node.Node('pictures', join(request.user.username, 'albums', albumid))
                album = AlbumObject.new(t, albumnode)
                album.index.pictures = albumlist
                root, ext = os.path.splitext(obj.filename)
                if ext == '.zip':
                    album.title = root  # FIXME: decode
                else:
                    pass # default is "New Album"

                album.album_thumb_pic = album.index.pictures[0]
                album.save(AddAlbumChangeset({
                            'message': 'Album created from file %s'%obj.index.filename,
                            'committer': request.user.userid(),
                            }))
                return HTTPFound('Archive uploaded.', location=request.node.url_to(albumnode, action='edit'))

            else:
                try:
                    picnode = self.create_picture(t, request, obj)
                except AlreadyExistsError, e:
                    return self.render_to_response(request, 'pictures/error.html', {
                            'error': mark_safe('This file already exists as <a href="%s">%s</a>.' % (request.node.url_to(e.existing), e.existing.path)),
                            })

        return HTTPFound('Picture uploaded.', location=request.node.url_to(path=picnode.path))

    def create_picture(self, t, request, fileobj):
        filename = fileobj.index.filename
        picid = "%d%06d"%(time.time(), datetime.datetime.now().microsecond)
        picnode = node.Node('pictures', join(request.user.username,picid))
        if t.exists(picnode):
            # This should NEVER EVER HAPPEN (vestigal code)
            raise AlreadyExistsError, ('This file already exists.', picnode)

        # FIXME: more elegant way to cache this -- i.e. implicitly cache
        # some kind of query -- rather than explicitly manage a cache?
        cache = PicturesCache.get(t)
        # FIXME: This probably makes everything much slower! Do we
        # really need to see whether uploading this image would
        # duplicate other image data?
        hash = hash_image(fileobj.data)
        if hash in cache.hash_to_image:
            raise AlreadyExistsError, ('This file exists already', cache.hash_to_image[hash])
        cache.hash_to_image[hash] = picnode

        picture = PictureObject.new(t, picnode)
        picture.filenode = fileobj.get_node()
        picture.read_exifinfo(fileobj)
        fileobj.lockersmeta.enclosed_by = picnode
        # FIXME: This is the only use of PRESERVE that I can find.
        # It's really just a hack so that I don't override the file
        # with an exact copy of the streamed file-like-object. There
        # should really be a check for this.
        if not fileobj.is_new():
            fileobj.data = PRESERVE
        fileobj.save({'message': 'uploaded file %s'% filename},
#                     data=lambda filename: self.jhead(filename)
                     )
        picture.save(AddPictureChangeset({
                    'message': 'Picture added: %s'% filename,
                    'committer': request.user.userid(),
                    }))
        return picnode

    def upload_form(self, request):
        return self.render_to_response(request, 'pictures/upload_form.html', {
                'form': PictureUploadFileForm()
                })

    def get_containing(self, t, pic_node):
        container_node = node.Node('cache', 'pictures/albums_containing', untracked=True)
        tracker = AlbumContainerTracker.get(t, container_node)
        if pic_node not in tracker.album_containing_picture:
            l = []
            # FIXME: multi-user
            for album_node in t.query().node(pic_node.replace(path='ethan/albums')).children().generate():
                a = AlbumObject(t, album_node)
                if pic_node in a.pictures:
                    l.append(album_node)

            tracker.album_containing_picture[pic_node] = l
            tracker.save()

        album_nodes = tracker.album_containing_picture[pic_node]
        result = []
        for album_node in album_nodes:
            a = AlbumObject(t, album_node)
            result.append({'node': album_node, 'title': a.title})
        return result

    def display_single(self, request, node):
        with self.backend.new_transaction() as t:
            p = PictureObject.get(t, node)
            title = 'a picture'
            g = None
            context = None
            contextname = ''
            src = request.GET.get('from', '')
            if src.startswith('album-'):
                albumpath = node.replace(module='pictures', path=join(p.get_user(), 'albums', src.replace('album-', '')))
                a = AlbumObject(t, albumpath)
                g = iter(a.pictures)

            if g == None:
                # FIXME: store the creator of the object?
                g = t.query().node(node.replace(path=p.get_user())).children(object=True).generate()

            prev, next = self.get_neighbors(p, g)
            prev = PictureObject.get_or_dummy(t, prev, 'prev')
            next = PictureObject.get_or_dummy(t, next, 'next')
            containing = self.get_containing(t, node)

        return self.render_to_response(request, 'pictures/single.html', {
                'picture': p,
                'title': title,
                'containing': containing,
                'nextpic': next, 'prevpic': prev,
                'fromstring': src.encode('utf-8') # FIXME: more sensible way to do this?
                })

    def get_neighbors(self, picture, generator):
        '''Find the previous and next pictures for the given picture, when
        iterating over the given query.'''
        node = picture.get_node()

        try:
            first = generator.next()
        except StopIteration, s:
            # this should never happen!
            raise ValueError, "no pictures match this query: %s"%(picture.get_node(),)
        if first == node:
            try:
                return [None, generator.next()]
            except StopIteration, s:
                return [None, None]
        while True:
            last = first
            try:
                first = generator.next()
            except StopIteration, s:
                break
            if first == node:
                try:
                    return [last, generator.next()]
                except StopIteration, s:
                    return [last, None]

        return [None, None]

    def all_gallery(self, request, node):
        # FIXME: much deeper magic
        return self.user_gallery('ethan', request, node)

        return self.render_to_response(request, 'pictures/gallery.html',
                                       {})

    def user_gallery(self, username, request, node, album=None):
        # FIXME: separate method for specific galleries? Join?
        NUMBER_PICTURES = 5
        NUMBER_ALBUMS = 5

        with self.backend.new_transaction() as t:
            q = t.query().node(node.replace(path=username)).children(object=True).sort(reverse=True).limit(NUMBER_PICTURES)
            recent_pics = [PictureObject(t, node) for node in q.generate()]

            q = t.query().node(node.replace(path=join(username, 'albums'))).children().sort(reverse=True).limit(NUMBER_ALBUMS)
            albums = [AlbumObject(t, node) for node in q.generate()]

            if not recent_pics:
                return self.render_to_response(request, 'pictures/gallery.html',
                                               {})

        return self.render_to_response(request, 'pictures/user_gallery.html', {
                'title': username+"'s photos",
                'album_list': albums,
                'picture_list': recent_pics,
                'username': username,
                # FIXME: blech
                'username_pics': join(username, 'pictures'),
                'username_albums': join(username, 'albums'),
                })

    def list_albums(self, request, node, username):
        NUMBER_ALBUMS = 10
        with self.backend.new_transaction() as t:
            q = t.query().node(node.replace(path=join(username, 'albums'))).children().sort(reverse=True)
            albums = []
            for node in q.generate():
                albums.append(node)

            paginator = pagination.Paginator(request.node, albums, NUMBER_ALBUMS, user_request=request.GET)

            objs = []
            for node in paginator.slice():
                objs.append(AlbumObject(t, node))

        return self.render_to_response(request, 'pictures/user_albums.html', {
                'username': username,
                'album_list': objs,
                'pagination': paginator,
                })

    def list_pictures(self, request, node, username):
        NUMBER_PICTURES = 25
        with self.backend.new_transaction() as t:
            q = t.query().node(node.replace(path=username)).children(object=True).sort(reverse=True)
            recent_pics = list(q.generate())

            paginator = pagination.Paginator(request.node, recent_pics, NUMBER_PICTURES, user_request=request.GET)

            objs = [PictureObject(t, node) for node in paginator.slice()]

        return self.render_to_response(request, 'pictures/user_pictures.html', {
                'username': username,
                'picture_list': objs,
                'pagination': paginator,
#                'pagination_previous': 'Newer pictures',
#                'pagination_next': 'Older pictures',
                })

    def display_album(self, request, node):
        with self.backend.new_transaction() as t:
            album = AlbumObject(t, node)
            pics = [PictureObject(t, p) for p in album.pictures]

        return self.render_to_response(request, 'pictures/album.html', {
                'title': album.title,
                'picture_list': pics,
                # FIXME: blech
                'fromstring': 'album-'+album.get_id(),
                })

    def history_album(self, request, node):
        # FIXME: if we ever add/subtract pictures from an album, this'll have
        # to be more complicated
        with self.backend.new_transaction() as t:
            album = AlbumObject.get(t, node)

            files = []
            for picnode in album.pictures:
                files.append(picnode)
                pic = PictureObject(t, picnode)
                files.append(pic.filenode)

            revids = t.history(files+[node])

            changesets = []
            lastrev = None
            revids.reverse()
            for rev in revids:
                c = Changeset.load(t, rev)
                c.last_revision = lastrev
                changesets.append(c)
                lastrev = rev
            changesets.reverse()

        return self.render_to_response(request, 'pictures/history_album.html', {
                'changesets': ChangesetRenderer(changesets, self.backend, request),
                'album': album,
                })

    @complain_if_revision
    @needs_auth
    def edit_album(self, request, node):
        if request.method == 'POST':
            # save a new version
            with self.backend.new_transaction() as t:
                obj = AlbumObject.get(t, node)

                # FIXME: fractally clumsy
                for i in range(len(obj.pictures)):
                    picnode = obj.pictures[i]
                    pic = PictureObject(t, picnode)
                    newcaption = request.POST.getone(u'album-pictures-%d-caption'%(i,))
                    # Factor this stuff out -- diffs should be done in one place
                    if unicode(pic.caption.raw()) != newcaption:
                        caption = {'old': pic.caption.raw(), 'new': newcaption}
                        pic.caption = newcaption
                        pic.exif = PRESERVE
                        pic.save(EditPictureChangeset({'caption': caption,
                                                       'message': request.POST.get('message'),
                                                       'committer': request.user.userid()}))


                oldthumb = obj.get_image()
                if request.POST.getone('thumbpic'):
                    obj.album_thumb_pic = obj.pictures[int(request.POST.getone('thumbpic'))]
                newthumb = obj.get_image()
                oldname = obj.title
                obj.title = request.POST.getone('title')
                newname = obj.title
                name_diff = None
                thumb_diff = None
                if newname != oldname:
                    name_diff = {'old': oldname, 'new': newname}
                if newthumb != oldthumb:
                    thumb_diff = {'old': oldthumb, 'new': newthumb}
                if name_diff or thumb_diff:
                    obj.save(EditAlbumChangeset({'message': request.POST.get('message', None),
                                                 'name': name_diff,
                                                 'thumb': thumb_diff,
                                                 'committer': request.user.userid(),
                                                 }))

            url = node.url_to()

            return HTTPFound(detail='Album saved.', location=url)

        with self.backend.new_transaction() as t:
            album = AlbumObject(t, node)
            if album.album_thumb_pic == '':
                albumpic = 0
            else:
                albumpic = album.pictures.index(album.album_thumb_pic)
            album.pictures = [PictureObject(t, p) for p in album.pictures]

        # FIXME: django.forms?
        return self.render_to_response(request, 'pictures/edit_album.html', {
                'album': album,
                'thumbpic': albumpic
                })

    def jpegtran_rot(self, node, degrees, cs):
        degrees = str(degrees)
        with self.backend.new_transaction() as t:
            # FIXME: if not JPEG?
            p = PictureObject(t, node)
            f = files.FileObject.get(t, p.filenode)
            stat, out, err = utils.pexecl('/usr/bin/jpegtran', 'jpegtran', '-rot', degrees, '-copy', 'all', p.filenode.part('data'))
            if stat: raise TypeError, 'jpegtran broke: %s %s %s'%(stat, out, err)
            f.data = out
            f.save(cs)


    @complain_if_revision
    @needs_auth
    def ajax_rotate(self, request, node):
        # FIXME: this should be a POST, not a GET.  But that's hard
        # because that means links need to become buttons to submit
        # forms (so as to be Hijax-compliant), and that sounds like a
        # lot of work.
        if request.GET.get('direction', 'cw') == 'cw':
            degrees = 90
        else:
            degrees = 270
        albumid = request.GET.get('album', None)
        if albumid:
            albumid = node.replace(path=albumid)

        cs = RotationChangeset({
                'committer': request.user.userid(),
                'album': albumid, 'degrees': degrees,
                'image': node,
                                })

        self.jpegtran_rot(node, degrees, cs)

        return Response('ok')

pictures = PicturesApplication
