from __future__ import with_statement
from lockers import application
import lockers.node
from lockers.format.rst import RST
from webob import Response
import webob.exc
from lockers.vcs import FileNotFoundError, VCSException
from lockers.models.models import Changeset, LockersObject, DoesNotExistError, ModelFactory, ChangesetFactory
from lockers.template import render_to_response
import lockers.user
from contrib.django import newforms

# We'll use this controller for doing actual user manipulation --
# /user/login, /user/logout, /user/profile?action=edit
#
# To view me, you'll have to do /users/Ethan or /users/0000001 or
# whatever.

import traceback

class UserChangeset_1(Changeset):
    pass

UserChangeset = ChangesetFactory('UserChangeset')

class LoginForm(newforms.Form):
    username = newforms.CharField()
    password = newforms.CharField(required=False, widget=newforms.PasswordInput)
#    next = newforms.CharField(widget=newforms.HiddenInput, required=False)

class UserApplication(application.LockersApplication):
    '''Lockers module for user manipulation.'''
    def __init__(self):
        super(UserApplication, self).__init__()
        self.rst = RST()

    def __call__(self, request):
        super(UserApplication, self).__call__(request)
        node = request.node

        if node.path == 'logout':
            lockers.user.logout(request)
            return webob.exc.HTTPFound(detail='Logged out.', location=request.referrer)

        if node.path == 'login':
            if request.method == 'POST':
                return self.login_submit(request)
            else:
                return self.login_form(request, next=request.referrer)

        return self.render_to_response(request, 'main.html', {
                'title': 'OK'
                })

    def login_form(self, request, reason=None, failed=None, next=None):
        form = LoginForm()
        if request.method == 'POST':
            form = LoginForm(request.POST)

        # FIXME: request.flash
        if reason == None: reason = request.GET.get('reason', None)

        # FIXME: fail more gracefully here?
        if next == None: next = request.POST['next']

        request.session['challenge'] = lockers.user.challenge()
        dct = {'challenge': request.session['challenge'],
               'salt': lockers.user.SALT,
               'form': form,
               'next': next,
               }
        if failed: dct['failed'] = failed
        if reason: dct['reason'] = reason

        return self.render_to_response(request, 'user/login.html', dct)

    def login_submit(self, request):
        form = LoginForm(request.POST)
        if form.is_valid():
            if request.user.is_authenticated():
                return webob.exc.HTTPFound(detail='Login OK.', location=request.POST['next'].encode('utf8'))


        return self.login_form(request, failed='Login incorrect. Try again.')


    def create(self, username, realname, password):
        # FIXME: next_child
        n = lockers.node.Node('users', '/0000001')
        with self.backend.new_transaction() as t:
            user = lockers.user.UserObject.new(t, n, username=username, 
                                       realname=realname)
            user.set_password(password)
            user.save({'message': 'Creating user ' + username})

user = UserApplication

if __name__ == '__main__':
    # invoke using something like:
    # python run.py --run "execfile('modules/user.py')" --create blah blah blah

    import sys
    if len(sys.argv) <= 2: # run.py --cmd
        print 'Usage: user.py --create username realname password'
        sys.exit()

    cmd = sys.argv[1]
    if cmd == '--create':
        username, realname, password = sys.argv[2:]
        UserApplication().create(username, realname, password)

