import sys
import os
import unittest
import random
import mimetypes
import urlparse

# Add the relevant stuff to sys.path.
# This code is taken from the docutils testing framework.
testroot = os.path.abspath(os.path.dirname(__file__) or os.curdir)
sys.path.insert(0, os.path.normpath(os.path.join(testroot, '..')))
sys.path.insert(0, os.path.normpath(os.path.join(testroot, '..', 'contrib')))
sys.path.insert(0, testroot)
sys.path.append(os.path.normpath(os.path.join(testroot, '..', 'extras')))
# FIXME: this is all garbage anyhow, see lockers.template
import lockers.django_conf

FAST = True
if 'thorough' in sys.argv:
    FAST = False
    sys.argv.remove('thorough')

import TestApplication
import unittest
from contrib import fixture
import webtest
import syck
from webob.exc import *

class TestApp(webtest.TestApp):
    '''
    Smarter testapp that lets you specify the content_types for files,
    without relying on mimetypes.guess_type.

    Programming by cut-and-paste abounds here.
    '''
    def __init__(self, *args, **kwargs):
        super(TestApp, self).__init__(*args, **kwargs)
        self._content_types = {}

    def set_content_type(self, filename, content_type):
        self._content_types[filename] = content_type

    def encode_multipart(self, params, files):
        '''
        Smarter multipart encoder that checks self._content_types for mimetypes,
        falling back to mimetypes.guess_type when needed.
        '''
        boundary = '----------a_BoUnDaRy%s$' % random.random()
        lines = []
        for key, value in params:
            lines.append('--'+boundary)
            lines.append('Content-Disposition: form-data; name="%s"' % key)
            lines.append('')
            lines.append(value)
        for file_info in files:
            key, filename, value = self._get_file_info(file_info)
            lines.append('--'+boundary)
            lines.append('Content-Disposition: form-data; name="%s"; filename="%s"'
                         % (key, filename))
            fcontent = self._content_types.get(filename)
            if fcontent == None:
                fcontent = mimetypes.guess_type(filename)
            lines.append('Content-Type: %s' %
                         (fcontent or 'application/octet-stream',))
            lines.append('')
            lines.append(value)
        lines.append('--' + boundary + '--')
        lines.append('')
        body = '\r\n'.join(lines)
        content_type = 'multipart/form-data; boundary=%s' % boundary
        return content_type, body

class TestCase(unittest.TestCase):
    def request_call_function(self, req, func):
        from lockers.session import SessionMiddleware
        s = SessionMiddleware(func)
        s(req)

# FIXME: didn't use Amara, because I don't want another dependency just to run
# tests. If we add it for other reasons, it might be OK.

import xml.etree.ElementTree as ET
def et_parse(str):
    parser = ET.XMLTreeBuilder()
    parser.entity['nbsp'] = unichr(160)
    try:
        parser.feed(str)
    except xml.parsers.expat.ExpatError, e:
        lines = re.split('\n|\r', str)
        print e, e.lineno, lines[e.lineno]
        raise
    tree = ET.ElementTree(parser.close())
    return tree

def et_get_parent(tree, node):
    for parent in tree.getiterator():
        if node in parent:
            return parent
    raise TypeError, 'node not in tree'

# FIXME: XML is hard; let's go shopping
def xhtml(elem):
    return '{http://www.w3.org/1999/xhtml}%s'%(elem,)

class ModuleTestCase(TestCase):
    def setUp(self):
        # Where the data directory goes
        if not hasattr(self, 'TESTDIRECTORY'):
            raise TypeError, 'ModuleTestCase needs a TESTDIRECTORY attribute'
        TestApplication.init(self.TESTDIRECTORY)
        # FIXME: name collision with global var
        import run
        self.lockers = run.LockersMain()
        self.testapp = TestApp(self.lockers)

    def login(self, username, password, location='/wiki/Start', path='/user/login'):
        # Largely the same as test_user's stuff, but un-modular.
        # FIXME: remove duplication?
        # FIXME: just do some voodoo WSGI magic, don't do this the hard way
        loginres = self.testapp.get(path, headers={'referer': location})
        loginform = loginres.forms[0]  # forms are stored under ids, not names. FIXME?
        loginform['username'] = username
        loginform['password'] = password
        result = loginform.submit()

        if result.status == HTTPFound.code or getattr(result, 'status_int', 0) == HTTPFound.code:
            result = result.follow()

        return result

    def assert_userid(self, uid):
        echo = self.testapp.get('/__echo')
        echo = syck.load(echo.body)
        self.assertEqual(echo.get('session', {}).get('userid', None), uid)

    def assert_anonymous(self):
        return self.assert_userid(None)

    def assert_needs_auth(self, res):
        self.assertEqual(res.status_int, HTTPFound.code)
        # FIXME: localhost?
        loc = res.headers['Location']
        loc = loc[:loc.index('?')]
        if loc not in ['http://localhost:80/user/login', '/user/login']:
            raise AssertionError, 'request does not redirect to a login page'

    def _testfile(self, name):
        return os.path.join(testroot, 'module_tests', 'test_data', name)

    def upload(self, filename, content_type, params, path='/files/upload', fieldname='file'):
        # FIXME: compare params against a given form?
        self.testapp.set_content_type(self._testfile(filename), content_type)
        res = self.testapp.post(path, params=params,
                                upload_files=[(fieldname, self._testfile(filename))])
        return res

    def assert_same_link(self, url1, url2):
        if url1 == None or url2 == None:
            self.assertEqual(url1, url2)
            return
        schema1, netloc1, path1, params1, query1, fragment1 = urlparse.urlparse(url1)
        schema2, netloc2, path2, params2, query2, fragment2 = urlparse.urlparse(url2)
        # schema and netloc are ignored; we aren't going to provide test cases
        # that have those!
        self.assertEqual([path1, params1, fragment1],
                         [path2, params2, fragment2])

        # now just have to compare query arguments
        def parse_query(q):
            qparms = query1.split('&')
            qhash = {}
            for parm in qparms:
                key, val = parm.split('=')
                # FIXME: %20 etc
                qhash.setdefault(key, []).append(val)
            return qhash
        self.assertEqual(parse_query(query1), parse_query(query2))

    def find_link(self, res, text):
        tree = et_parse(res.body)
        for node in tree.getiterator():
            if node.text and text in node.text:
                return node.attrib['href']

import lockers.vcs
import lockers.backend

class TestVCS(lockers.vcs.VCS):
    def __init__(self, *files):
        self.files = files

    def add_files(self, *files):
        self.files += files

    def _current_revision(file):
        return '1'

    def _get(self, filename, revid):
        if not revid: revid = ''
        if filename in self.files:
            return filename+':'+revid
        else:
            raise lockers.vcs.VCSException, "file does not exist"

    def _commit_message(self, revid):
        return revid

    def _history(self, filenames):
        return '1'

    def _listdir(self, directory, revid):
        return [f for f in self.files if f.startswith(directory)]

    def _isdir(self, filename, revid):
        return len([f for f in self.files if f.startswith(filename)]) != 0

def run(name):
    if name == '__main__':
        unittest.main()
