'''Data set including a few pages.

rev1
  wiki/
    Taggable/Obj1 [node0]
        data1 -> 'hi there'
        data2 -> 14

rev2
  wiki/
    Taggable/Obj1 [node1]
        data1 -> 'foo'

rev3
  wiki/
    Taggable/Obj2 [node2]
        data1 -> 'bar'
        data2 -> 42

'''
from __future__ import with_statement
from lockers.models.models import LockersObject, Changeset, ModelFactory
from lockers.models.fields import NumField, StrField
from lockers.node import Node

class RandomObject_1(LockersObject):
    data1 = StrField()
    data2 = NumField()
    key = StrField()

RandomObject = ModelFactory('RandomObject')

populated = False

node0 = None # only populated later
node1 = Node('wiki', 'Taggable/Obj1')
node2 = Node('wiki', 'Taggable/Obj2')

objects = []
def populate(backend):
    global node0, objects
    if hasattr(backend, 'populated_dataset1'): return objects
    backend.populated_dataset1 = True
    objects = []

    with backend.new_transaction() as t:
        obj0 = RandomObject.new(t, node1)
        obj0.data1 = 'hi there'
        obj0.data2 = 14
        obj0.save(Changeset({'message': 'Create object'}))

    node0 = node1.replace(revision=backend.current_revision())

    with backend.new_transaction() as t:
        obj1 = RandomObject.get(t, node1)
        obj1.data1 = 'foo'

        obj1.save(Changeset({'message': 'Add foo'}))

        obj2 = RandomObject.new(t, node2)
        obj2.data1 = 'bar'
        obj2.data2 = 42

        obj2.save(Changeset({'message': 'Another object'}))

    objects.extend([obj1, obj2])

    return objects
    
