'''
files/
   000000000   # len -> 9
   ...
   000000014
'''
from __future__ import with_statement
from lockers.models.models import Changeset
from lockers.node import Node
from lockers.user import UserObject
USER1_PW = 'password'
USER2_PW = '1122!!$$$oU'

def run(backend):
    if not hasattr(backend, 'd3_populated'):
        populate(backend)
    return (backend.d3_nodes, backend.d3_users, backend.d3_passwords,
            backend.d3_revisions)

def populate(backend):
    backend.d3_populated = True
    def add_user(t, path, username, realname, pw, message):
        node = Node('users', '/'+path)
        user = UserObject.new(t, node, username=username,
                                   realname=realname)
        user.set_password(pw)
        user.save(Changeset({'message': message}))
        return [node, user]

    with backend.new_transaction() as t:
        uid1 = '0000001'
        node1, user1 = add_user(t, uid1, 'barbie', 'Barber E', USER1_PW,
                                'Creating a user')

    rev1 = backend.current_revision()

    with backend.new_transaction() as t:
        uid2 = '0000002'
        node2, user2 = add_user(t, uid2, 'bubbles', 'J.Random user',
                                USER2_PW, 'Creating user2')

    rev2 = backend.current_revision()

    backend.d3_userids = [uid1, uid2]
    backend.d3_users = [user1, user2]
    backend.d3_nodes = [node1, node2]
    backend.d3_revisions = [rev1, rev2]
    backend.d3_passwords = [USER1_PW, USER2_PW]
