from __future__ import with_statement
from __init__ import TestSupport
import run
from lockers import application
from lockers.node import Node
from lockers import backend

from modules import wiki

from test import TestApplication
from webob.exc import *
import dataset3
import re
import os.path

from TestSupport import TestApp, ModuleTestCase

class FileTests(ModuleTestCase):
    location = '/wiki/Start'
    TESTDIRECTORY = '/tmp/filestest'
    def setUp(self):
        super(FileTests, self).setUp()

        self.backend = backend.get_backend()
        dataset3.run(self.backend)

        self.USER1_PW, self.USER2_PW = self.backend.d3_passwords[:2]
        self.user1, self.user2 = self.backend.d3_users[:2]
        self.uid1, self.uid2 = self.backend.d3_userids[:2]

        # FIXME: home page
        n = Node('wiki', '/Start')
        with self.backend.new_transaction() as t:
            if not t.exists(n):
                obj = wiki.WikiObject.new(t, n)
                obj.index = 'Hi'
                obj.save({'message': 'Get rid of warnings'})

    def login(self):
        return super(FileTests,self).login('barbie', self.USER1_PW)

    def get_contents(self, filename):
        return file(self._testfile(filename)).read()

    def check_upload_file(self, filename, description, content_type):
        res = self.testapp.get('/files/upload')
        self.assertEqual(res.status_int, 200)
        form = res.forms[0]
        # FIXME: Interact with the form in some way? Doesn't seem to have
        # support for File objects..
        textres = self.upload(filename, content_type, {'description': description})
        fileloc = textres.follow()
        fileloc.mustcontain(description)
        newtext = fileloc.click('Download this file')
        assert newtext.headers['Content-Type'] == content_type or newtext.headers['Content-Type'].startswith(content_type + ';'), (
            'got a bad mimetype: %s (expected %s)'%(newtext.headers['Content-Type'], content_type))

        self.assertEqual(newtext.body, self.get_contents(filename))

    def test_uploads(self):
        self.login()
        self.check_upload_file('textfile.txt', 'Testing plaintext upload',
                               'text/plain')
        self.check_upload_file('bitmap.bmp', 'Testing binary upload',
                               'image/bmp')
        self.check_upload_file('photo.jpg', 'Testing photos',
                               'image/jpeg')  # FIXME: EXIF as well as JFIF


    def test_upload_file_anonymous(self):
        # FIXME: anonymous users can't upload files?
        res = self.testapp.get('/files/upload')


TestSupport.run(__name__)
