from __future__ import with_statement

from __init__ import TestSupport
import os.path
import PIL.Image
import run
from lockers import application
from lockers.node import Node
from lockers import backend
from lockers import utils
from lockers import archive

from modules import wiki

from test import TestApplication
from TestSupport import et_parse, et_get_parent, xhtml
import dataset3

import re
import xml.parsers.expat

def tree_to_thumb_links(tree):
    l = []
    for img in tree.findall('//%s/%s'%(xhtml('a'), xhtml('img'))):
        parent = et_get_parent(tree, img)
        if parent.tag != xhtml('a'): continue   # can't happen

        if img.attrib['src'].startswith('http://localhost:80/pictures/') and \
           parent.attrib['href'].startswith('http://localhost:80/pictures/'):
            l.append((parent.attrib['href'], img.attrib['src']))
    return l

# FIXME: this whole test suite is huge, bloated, only three tests,
# basically just sucks, lots of repeated code
def thumb_to_pictures_image(res):
    tree = et_parse(res.body)

    return [link for link, img in tree_to_thumb_links(tree) if
            '/albums/' not in link]

def thumb_to_pictures_thumb(res):
    tree = et_parse(res.body)

    return [img for link, img in tree_to_thumb_links(tree) if
            '/albums/' not in link]

def thumbs_to_albums(res):
    tree = et_parse(res.body)

    return [link for link, img in tree_to_thumb_links(tree) if
            '/albums/' in link]

def thumbs_for_albums(res):
    tree = et_parse(res.body)
    return [img for link, img in tree_to_thumb_links(tree) if
            '/albums/' in link]

class PicturesTests(TestSupport.ModuleTestCase):
    TESTDIRECTORY = '/tmp/picturestest'
    def setUp(self):
        super(PicturesTests, self).setUp()
        self.backend = backend.make_backend()
        dataset3.run(self.backend)
        self.username = 'barbie'
        self.password = self.backend.d3_passwords[0]

    def test_no_pictures(self):
        res = self.testapp.get('/pictures')
        assert 'No pictures' in res
        assert not thumb_to_pictures_image(res)


    def navigation(self, uri):
        res = self.testapp.get(uri)
        tree = et_parse(res.body)
        # FIXME: this utterly sucks
        for node in tree.getiterator():
            if node.text and 'Browse' in node.text:
                browselinks = et_get_parent(tree, node).getchildren()[1:]
                # Remove last link if it's back to the album
                if browselinks[-1].tag == xhtml('a'):
                    del browselinks[-1]
                return browselinks

    def no_prev(self, uri):
        prev, next = self.navigation(uri)
        if prev.getchildren()[0].tag != xhtml('img'):
            raise AssertionError, 'a previous image was found for %s: %s'%(uri, prev.getchildren()[0].get('href'))
        src = prev.getchildren()[0].get('src')
        assert 'no_previous' in src
        res = self.testapp.get(src)

    def no_next(self, uri):
        prev, next = self.navigation(uri)
        if next.getchildren()[0].tag != xhtml('img'):
            raise AssertionError, 'a next image was found for %s: %s'%(uri, next.getchildren()[0].get('href'))
        src = next.getchildren()[0].get('src')
        assert 'no_next' in src
        res = self.testapp.get(src)

    def has_prev(self, uri):
        prev, next = self.navigation(uri)
        assert prev.getchildren()[0].tag == xhtml('a')

    def has_next(self, uri):
        prev, next = self.navigation(uri)
        assert next.getchildren()[0].tag == xhtml('a')

    def no_albums(self, res):
        self.assertEqual(thumbs_to_albums(res), [])

    def assert_contiguous(self, pic_urls):
        # FIXME: assert that pic_urls[0].next -> pic_urls[1], and backwards
        self.no_prev(pic_urls[0])
        self.has_next(pic_urls[0])
        for pic in pic_urls[1:-1]:
            self.has_next(pic)
            self.has_prev(pic)
        self.has_prev(pic_urls[-1])
        self.no_next(pic_urls[-1])

    def get_albuminfos(self):
        res = self.testapp.get('/pictures/barbie')
        albumthumbs = thumbs_for_albums(res)
        albumlinks = thumbs_to_albums(res)
        l = []
        for img, link in zip(albumthumbs, albumlinks):
            d = {'thumb': img, 'link': link}
            res = self.testapp.get(link)
            match = re.search('<title>[^<]*: ([^<]+)</title>', res.body)
            if not match:
                raise IOError, "Couldn't find title of album in body: %s"%(res.body)
            title = match.groups()[0].strip()
            d['title'] = title
            l.append(d)

        return l

    def test_0_upload_pictures(self):  # funny name to make it run first
        self.assert_anonymous()
        def upload1():
            return self.upload('photo.jpg', None, {'description': 'Test photo 1'}, '/pictures/upload')

        self.assert_needs_auth(upload1())

        res = self.login(self.username, self.password, location='/pictures/upload')
        res = upload1()

        res = self.testapp.get('/pictures/barbie')
        assert 'No pictures' not in res
        self.no_albums(res)
        pic_urls = thumb_to_pictures_image(res)
        assert pic_urls != []
        pic1 = pic_urls[0]

        self.no_prev(pic1)
        self.no_next(pic1)

        def upload2():
            return self.upload('photo2.jpg', None, {'description': 'Test photo 2'}, '/pictures/upload')

        res = upload2()
        res = self.testapp.get('/pictures/barbie')
        assert 'No pictures' not in res
        pic_urls = thumb_to_pictures_image(res)
        assert len(pic_urls) == 2
        assert pic1 in pic_urls
        pic_without_1 = [pic for pic in pic_urls if pic != pic1]
        assert len(pic_without_1) == 1
        pic2 = pic_without_1[0]
        both_pics = pic_urls

        res = upload1()
        assert re.search('already exists as [^<]*<a href="'+pic1, res.body)

        res = self.testapp.get('/pictures/barbie')
        assert 'No pictures' not in res
        pic_urls = thumb_to_pictures_image(res)
        self.assertEqual(both_pics, pic_urls)
        self.assertEqual(len(pic_urls), 2)
        # This is kind of crummy, but it's a reverse-chron assertion
        self.assertEqual(pic_urls, [pic2, pic1])


        self.no_prev(pic1)
        self.has_next(pic1)
        self.no_next(pic2)
        self.has_prev(pic2)

        self.no_albums(res)

        res = self.upload('test_album.zip', 'application/octet-stream', {'description': 'Testing albums', 'unpack': True}, '/pictures/upload')
        # Test that it redirects somewhere sensible
        res = res.follow()


        res = self.testapp.get('/pictures/barbie')
        assert 'No pictures' not in res
        pic_urls = thumb_to_pictures_image(res)
        self.assertEqual(len(pic_urls), 5)

        album_urls = thumbs_to_albums(res)
        self.assertEqual(len(album_urls), 1)
        album1 = album_urls[0]
        albumid = album1.split('/')[-1]

        assert pic1 in pic_urls
        assert pic2 in pic_urls
        allpics = pic_urls
        #print 'pic1 is', pic1, 'pic2 is', pic2
        #print 'all pics are', allpics

        # More crumminess that relies on reverse-chron ordering.
        # Don't get me wrong, we want to guarantee reverse-chron, but it seems
        # like there's a right way and a wrong way to guarantee that.
        # FIXME: maybe nicer techniques using queries on the repo?
        self.assertEqual(allpics[-1], pic1)
        self.assertEqual(allpics[-2], pic2)
        pic5 = allpics[0]

        # allpics is [pic5, pic4, pic3, pic2, pic1]
        self.assert_contiguous(allpics[::-1])

        res = self.testapp.get(album1)
        pic_urls = thumb_to_pictures_image(res)
        self.assertEqual(len(pic_urls), 3)
        for albumpic, origpic in zip(pic_urls, allpics[2::-1]):
            self.assertEqual(albumpic, origpic+'?from=album-'+albumid)

        self.assert_contiguous(pic_urls)

        # Test zip
        # FIXME: routes? find the link from HTML?
        res = self.testapp.get(album1+'?action=download-zip')
        fname = utils.make_tempfile('test-pictures-got-zip')
        file(fname, 'w').write(res.body)
        a = archive.open(res.content_type, fname, fname)
        self.assertEqual(a.listfiles(), ['photo3.JPG', 'photo4.JPG', 'photo5.JPG'])
        assert res.headers.get('content-disposition').endswith('filename="test_album.zip"')

        # Test editing
        albuminfos = self.get_albuminfos()
        self.assertEqual(albuminfos, [{'thumb': allpics[2]+'?action=thumb0',
                                       'title': 'test_album',
                                       'link': album1}])

        res = self.testapp.get(album1+'?action=edit')
        form = res.forms[0]
        form['title'] = 'My First Album'
        form['thumbpic'] = 2
        res = form.submit()

        albuminfos = self.get_albuminfos()
        self.assertEqual(albuminfos, [{'thumb': pic5+'?action=thumb0',
                                       'title': 'My First Album',
                                       'link': album1}])

    def test_needs_auth(self):
        pic1 = self.latest_pic(thumb_to_pictures_image)
        self.assert_needs_auth(self.testapp.get(pic1+'?action=ajax-rotate&direction=cw'))

        album1 = self.latest_album()
        self.assert_needs_auth(self.testapp.get(album1+'?action=edit'))

        self.assert_needs_auth(self.testapp.post(album1+'?action=edit'))


    def latest_pic(self, f=thumb_to_pictures_thumb):
        # Still reverse-chron but at least it's separate
        res = self.testapp.get('/pictures/barbie')
        assert 'No pictures' not in res
        pic_urls = f(res)
        return pic_urls[0]

    def latest_album(self, f=thumbs_to_albums):
        res = self.testapp.get('/pictures/barbie')
        album_urls = f(res)
        return album_urls[0]

    def all_pictures(self, url='/pictures/barbie/pictures', f=thumb_to_pictures_thumb):
        res = self.testapp.get(url)
        return f(res)

    def get_pagination_links(self, url):
        res = self.testapp.get(url)
        tree = et_parse(res.body)
        if not tree.findall('//%s'%(xhtml('ul'),)):
            return []

        links = []
        for list in tree.findall('//%s'%(xhtml('ul'),)):
            for list_element in list:
                if len(list_element) != 1:
                    raise TypeError, 'HTML changed'
                node = list_element[0]
                if node.tag == xhtml('a'):
                    link = node.attrib['href']
                else:
                    link = None
                text = node.text
                links.append({'link': link,
                              'text': node.text})
        return links

    def image_dimensions(self, s):
        # FIXME: mktemp or something?
        fname = 'foo.jpg'
        fname = os.path.join('/tmp', fname)
        f = file(fname, 'w')
        f.write(s)
        f.close()

        i = PIL.Image.open(fname)
        x0, y0, w, h = i.getbbox()
        return (w, h)

    def assertAlbumSize(self, album, realw, realh):
        sep = '?'
        if sep in album: sep = '&'
        res = self.testapp.get(album+sep+'action=download-zip')
        fname = '/tmp/foo.zip'
        file('/tmp/foo.zip', 'w').write(res.body)
        a = archive.open(res.content_type, fname, fname)
        self.assertEqual(a.listfiles(), ['photo3.jpg'])
        picname = '/tmp/foo.jpg'
        contents = a.extract_as('photo3.jpg', picname)
        i = PIL.Image.open(picname)
        x0, y0, w, h = i.getbbox()
        self.assertEqual((w, h), (realw, realh))

    def test_rotate(self):
        res = self.login(self.username, self.password, location='/pictures/upload')

        #pic0 = self.latest_pic()   # Might not be any images yet

        res = self.upload('test_album2.zip', 'application/octet-stream', {'description': 'Testing rotation via albums', 'unpack': True}, '/pictures/upload')

        pic1 = self.latest_pic(thumb_to_pictures_image)
        pic1thumb = self.latest_pic()
        res = self.testapp.get(pic1thumb)

        w, h = self.image_dimensions(res.body)
        self.assertEqual((w, h), (200, 150))

        album1 = self.latest_album()
        album1thumb = self.latest_album(thumbs_for_albums)
        res = self.testapp.get(album1thumb)

        w, h = self.image_dimensions(res.body)
        self.assertEqual((w, h), (200, 150))

        self.assertAlbumSize(album1, 250, 188)


        # --- ROTATE! ---
        before_rot_rev = self.backend.current_revision()
        res = self.testapp.get(pic1+'?action=ajax-rotate&direction=cw')
        # FIXME: who knows what AJAX routines will return?
        self.assertEqual(res.body, 'ok')

        # FIXME: test that it actually rotated clockwise? Yeah right!
        res = self.testapp.get(pic1thumb)

        w, h = self.image_dimensions(res.body)
        self.assertEqual((w, h), (150, 200))   # Rotated!

        res = self.testapp.get(pic1thumb+'&revision='+before_rot_rev)
        w, h = self.image_dimensions(res.body)
        self.assertEqual((w, h), (200, 150))   # History!

        self.assertAlbumSize(album1, 188, 250)
        self.assertAlbumSize(album1+'?revision='+before_rot_rev, 250, 188)

    def test_history_urls(self):
        picsurl = '/pictures/barbie/pictures'
        allpics = self.all_pictures(picsurl, thumb_to_pictures_image)
        pic1 = allpics[0]
        rev1 = self.backend.current_revision()

        res = self.testapp.get(pic1+'?action=ajax-rotate&direction=cw')

        rev2 = self.backend.current_revision()

        res = self.testapp.get(pic1+'?action=ajax-rotate&direction=cw')

        rev3 = self.backend.current_revision()

        newpics = self.all_pictures(picsurl, thumb_to_pictures_image)
        self.assertEqual(pic1, newpics[0])

        oldpics = self.all_pictures(picsurl+'?revision='+rev1, thumb_to_pictures_image)
        self.assert_same_link(oldpics[0], pic1+'?revision='+rev1)

        oldprev, oldnext = self.navigation(oldpics[1])
        self.assert_same_link(oldpics[2], oldprev[0].attrib['href'])
        self.assert_same_link(oldpics[0], oldnext[0].attrib['href'])

    def test_edit_captions(self):
        picsurl = '/pictures/barbie'
        res = self.login(self.username, self.password, location=picsurl)
        res = self.testapp.get(picsurl)
        album_urls = thumbs_to_albums(res)
        res = self.testapp.get(album_urls[0])
        assert not "My <em>sexy</em> description" in res.body
        edit_link = self.find_link(res, 'Edit')
        edit_page = self.testapp.get(edit_link)
        edit_form = edit_page.forms[0]
        edit_form['album-pictures-0-caption'] = 'My *sexy* description'
        edit_form['message'] = 'Adding a description'
        edit_form.submit()

        res = self.testapp.get(album_urls[0])
        assert "My <em>sexy</em> description" in res.body

        edit_page = self.testapp.get(edit_link)
        edit_form = edit_page.forms[0]
        caption_form = edit_form.fields['album-pictures-0-caption'][0]
        self.assertEqual(caption_form.value, 'My *sexy* description')

    def compare_links(self, url, ideal):
        '''Used to compare pagination links on page specified by url against a specification'''
        app_url = url
        real = self.get_pagination_links(url)
        self.assertEqual(len(real), len(ideal))
        for num in range(len(real)):
            reallink = real[num]
            ideallink = ideal[num]
            self.assertEqual(reallink['text'], ideallink['text'])
            url = reallink['link']
            try:
                self.assert_same_link(url, ideallink['link'])
            except AssertionError, e:
                print self.testapp.get(app_url).body
                raise AssertionError, e.message + ' on link ' + reallink['text']

    def test_zzzz_pagination(self):
        picsurl = '/pictures/barbie/pictures'
        allpics = self.all_pictures(picsurl)
        n = len(allpics)
        # FIXME: use another user, upload exactly six pictures
        self.assertEqual(n, 6, 'test is too brittle to run with more/less than 6 pictures (got %s)'%(n,))
        pagination_links = self.get_pagination_links(picsurl)
        self.assertEqual(pagination_links, [])

        def create_numskip_link(num=None, skip=None):
            '''Helper in case we have to change all these test cases'''
            url = picsurl
            parts = []
            if num:
                parts.append('num='+str(num))
            if skip:
                parts.append('skip='+str(skip))
            if parts: return url + '?' + '&'.join(parts)
            return url
        numskip = create_numskip_link

        url = numskip(num=2)
        page1 = self.all_pictures(url)
        self.assertEqual(allpics[:2], page1)
        self.compare_links(url, [
                {'text': 'Previous', 'link': None},
                {'text': '1', 'link': None},
                {'text': '2', 'link': numskip(num=2, skip=2)},
                {'text': '3', 'link': numskip(num=2, skip=4)},
                {'text': 'Next', 'link': numskip(num=2, skip=2)}
                ])


        url = numskip(num=2, skip=2)
        page2 = self.all_pictures(url)
        self.assertEqual(allpics[2:4], page2)
        self.compare_links(url, [
                {'text': 'Previous', 'link': numskip(num=2, skip=0)},
                {'text': '1', 'link': numskip(num=2, skip=0)},
                {'text': '2', 'link': None},
                {'text': '3', 'link': numskip(num=2, skip=4)},
                {'text': 'Next', 'link': numskip(num=2, skip=2)}
                ])


        url = numskip(num=2, skip=1)
        page1_5 = self.all_pictures(url)
        self.assertEqual(allpics[1:3], page1_5)
        self.compare_links(url, [
                {'text': 'Previous', 'link': numskip(num=2, skip=0)},
                {'text': '1', 'link': numskip(num=2, skip=0)},
                {'text': '2', 'link': None},
                {'text': '3', 'link': numskip(num=2, skip=4)},
                {'text': 'Next', 'link': numskip(num=2, skip=3)}
                ])

        url = numskip(num=2, skip=4)
        page3 = self.all_pictures(url)
        self.assertEqual(allpics[4:6], page3)
        self.compare_links(url, [
                {'text': 'Previous', 'link': numskip(num=2, skip=2)},
                {'text': '1', 'link': numskip(num=2, skip=0)},
                {'text': '2', 'link': numskip(num=2, skip=2)},
                {'text': '3', 'link': None},
                {'text': 'Next', 'link': None}
                ])

        url = numskip(num=2, skip=3)
        page2_5 = self.all_pictures(url)
        self.assertEqual(allpics[3:5], page2_5)
        self.compare_links(url, [
                {'text': 'Previous', 'link': numskip(num=2, skip=1)},
                {'text': '1', 'link': numskip(num=2, skip=0)},
                {'text': '2', 'link': numskip(num=2, skip=2)},
                {'text': '3', 'link': None},
                {'text': 'Next', 'link': numskip(num=2, skip=5)}
                ])

    def test_zzzzzz_upload_directory(self):
        res = self.login(self.username, self.password, location='/pictures/barbie')
        res = self.upload('combined.zip', 'application/octet-stream', {'description': 'Testing rotation via albums', 'unpack': True}, '/pictures/upload')
        self.assertEqual(res.status_int, 302)


TestSupport.run(__name__)
