from __future__ import with_statement

from __init__ import TestSupport
import run
from lockers import application
from lockers.node import Node
from lockers import backend

from modules import wiki

from test import TestApplication
from webob.exc import *
import dataset3
import re

class UserTests(TestSupport.ModuleTestCase):
    location = '/wiki/Start'
    TESTDIRECTORY = '/tmp/usertest'
    def setUp(self):
        super(UserTests, self).setUp()

        self.backend = backend.get_backend()
        dataset3.run(self.backend)

        # FIXME: get userdata objects instead of parallel arrays or something
        self.USER1_PW, self.USER2_PW = self.backend.d3_passwords[:2]
        self.user1, self.user2 = self.backend.d3_users[:2]
        self.uid1, self.uid2 = self.backend.d3_userids[:2]

        n = Node('wiki', '/Start')
        with self.backend.new_transaction() as t:
            if not t.exists(n):
                obj = wiki.WikiObject.new(t, n)
                obj.index = 'Hi'
                obj.save({'message': 'Get rid of warnings'})

    def assert_success(self, res):
        self.assertEqual(res.status_int, HTTPFound.code)
        self.assertEqual(res.headers['Location'], 'http://localhost'+self.location) # FIXME
        assert not self.contains_error(res)
        res = res.follow()
        assert 'Hi barbie' in res
        assert not self.contains_error(res)
        self.assert_userid(self.uid1)

    def start_login(self, path='/user/login'):
        res = self.testapp.get(path, headers={'referer': self.location})
        loginform = res.forms[0]  # forms are stored under ids, not names. FIXME?
        return loginform

    def test_login_plaintext_success(self):
        self.assert_anonymous()
        # FIXME: doesn't validate JS
        loginform = self.start_login()
        res = self.succeed_plaintext(loginform)
        self.assert_success(res)

    def fail_plaintext(self, loginform):
        # FIXME: get username from dataset too
        loginform['username'] = 'barbie'
        loginform['password'] = self.USER2_PW
        return loginform.submit()

    def succeed_plaintext(self, loginform):
        loginform['username'] = 'barbie'
        loginform['password'] = self.USER1_PW
        return loginform.submit()

    def contains_error(self, response):
        body = response.body
        return re.compile('<div class="error">').search(body)

    def test_login_plaintext_failure(self):
        self.assert_anonymous()
        loginform = self.start_login()
        res = self.fail_plaintext(loginform)

        assert self.contains_error(res)
        self.assert_anonymous()

    def test_login_reason(self):
        # FIXME: flash
        res = self.testapp.get('/user/login?reason=XYZXYZXYZ', headers={'referer': self.location})
        assert re.compile('<div [^>]*>XYZXYZXYZ</div>').search(res.body)
        for i in range(2):
            res = self.fail_plaintext(res.forms[0])
            assert re.compile('<div [^>]*>XYZXYZXYZ</div>').search(res.body)
            assert self.contains_error(res)

        res = self.succeed_plaintext(res.forms[0])
        self.assert_success(res)

TestSupport.run(__name__)
