from __future__ import with_statement
import syck

from __init__ import TestSupport
import run
from lockers import application
from lockers.node import Node
from lockers import backend

from contrib import fixture
from modules import wiki

from test import TestApplication

class WikiTests(TestSupport.TestCase):
    def setUp(self):
        TestApplication.init('/tmp/wikitest')
        self.lockers = run.LockersMain()
        self.testapp = fixture.TestApp(self.lockers)
        self.backend = backend.make_backend()

        n = Node('wiki', '/Series/Baccano!')
        self.magictext = '123aooo456789' #unlikely to appear by chance
        with self.backend.new_transaction() as t:
            if not t.exists(n):
                obj = wiki.WikiObject.new(t, n)
                obj.index = self.magictext
                obj.save({'message': 'test commit'})

    def test_save(self):
        res = self.testapp.get('/wiki/memes/Series_of_Tubes?action=edit')
        form = res.forms['editform']
        form['content'] = 'Hi there'
        form['message'] = 'commit 1'
        form.submit()

        n = Node('wiki', '/memes/Series_of_Tubes')
        self.assertEqual(self.backend.history(n), [self.backend.current_revision()])
        self.assertEqual(self.backend.get_part(n, 'index'), syck.dump('Hi there'))
        #self.testapp.get('/wiki/memes/Series_of_Tubes')
        
    def test_get(self):
        res = self.testapp.get('/wiki/Series/Baccano!')
        self.assert_(self.magictext in res)

    def test_get_nonexistent(self):
        # This could change at any moment
        pass

    def test_save_over(self):
        n = Node('wiki', '/Series/Baccano!')
        with self.backend.new_transaction() as t:
            history = t.history(n)

        res = self.testapp.get('/wiki/Series/Baccano!?action=edit')
        form = res.forms['editform']
        form['content'] = self.magictext + ' and then some'
        form['message'] = 'committery'
        form.submit()

        with self.backend.new_transaction() as t:
            newhist = t.history(n)
            self.assertEqual(len(history) + 1, len(newhist))
            self.assertEqual(t.get_part(n, 'index'), syck.dump(self.magictext + ' and then some'))

    def test_diff(self):
        # How to test if a successful diff was computed?
        # The format here can also change at any moment
        pass

TestSupport.run(__name__)
