from __future__ import with_statement

import TestSupport
import TestApplication

from webob import Response
import lockers.user as user
import lockers.backend as backend
from lockers.application import needs_auth, complain_if_revision
from lockers.request import LockersRequest
from lockers.session import SessionMiddleware
from lockers.node import Node
from lockers.user import UserObject
import dataset3


class DecoratorTests(TestSupport.ModuleTestCase):
    def setUp(self):
        if not hasattr(self, 'users_setup'):
            self.users_setup = True

            TestApplication.init('/tmp/backendtest')
            self.backend = backend.get_backend()
            dataset3.run(self.backend)
            self.USER1_PW, self.USER2_PW = self.backend.d3_passwords[:2]
            self.user1, self.user2 = self.backend.d3_users[:2]

    def setup_request(self, url='/foo/bar'):
        req = LockersRequest.blank(url)
        req.node = Node(req)
        def init_user(req):
            with self.backend.new_transaction() as t:
                user = UserObject.get(t, req)
                req.user = user
                return Response('OK')
        # Using SessionMiddleware to fill in the anonymous user.
        self.request_call_function(req, init_user)
        return req

    @needs_auth
    def my_fake_bound_function(self, request):
        return 42

    def test_needs_auth(self):
        @needs_auth
        def my_fake_function(request):
            return 45

        req = self.setup_request()
        self.assert_needs_auth(my_fake_function(req))
        self.assert_needs_auth(self.my_fake_bound_function(req))

        req.user = self.user1
        self.assertEqual(self.my_fake_bound_function(req), 42)
        self.assertEqual(my_fake_function(req), 45)

    @complain_if_revision
    def my_fake_bound_complain(self, request):
        return 42

    def test_complain_if_revision(self):
        @complain_if_revision
        def my_fake_complain(request):
            return 45

        req = self.setup_request('/foo/bar?revision=13')
        self.assertRaises(ValueError, my_fake_complain, req)
        self.assertRaises(ValueError, self.my_fake_bound_complain, req)

        req = self.setup_request('/foo/bar')
        self.assertEqual(my_fake_complain(req), 45)
        self.assertEqual(self.my_fake_bound_complain(req), 42)

TestSupport.run(__name__)
