import os, os.path
import TestSupport
from TestSupport import testroot
from lockers import archive

def datadir(filename):
    return os.path.join(testroot, 'test_data', filename)

class VCSTests(TestSupport.TestCase):
    FILE1 = 'this is file1\n'
    FILE2 = 'this is file2\n'
    def setUp(self):
        os.system("rm -rf /tmp/archivetest")
        try:
            os.makedirs('/tmp/archivetest')
        except OSError:
            pass

    def open_archive(self, filename):
        return archive.open('application/octet-stream', filename, filename)

    def empty(self):
        return not os.listdir('/tmp/archivetest')

    def test_zip(self):
        assert self.empty()
        arc = self.open_archive(datadir('test_zip.zip'))
        files = arc.listfiles()
        files.sort()
        self.assertEqual(files, ['file1.txt', 'file2.txt'])
        arc.extract_as('file1.txt', '/tmp/archivetest/file1.txt')
        self.assertEqual(open('/tmp/archivetest/file1.txt').read(),
                         self.FILE1)

        arc.extract_as('file2.txt', '/tmp/archivetest/boooo.txt')
        self.assertEqual(open('/tmp/archivetest/boooo.txt').read(),
                         self.FILE2)

    def test_zip_subdir(self):
        assert self.empty()
        arc = self.open_archive(datadir('test_subdir.zip'))

        files = arc.listfiles()
        files.sort()
        self.assertEqual(files, ['some_dir/', 'some_dir/file1.txt', 'some_dir/file2.txt'])
        arc.extract_as('some_dir/file1.txt', '/tmp/archivetest/file1.txt')
        self.assertEqual(open('/tmp/archivetest/file1.txt').read(),
                         self.FILE1)

        arc.extract_as('some_dir/file2.txt', '/tmp/archivetest/boooo.txt')
        self.assertEqual(open('/tmp/archivetest/boooo.txt').read(),
                         self.FILE2)

        self.assertRaises(IOError, lambda: arc.extract_as('some_dir/', '/tmp/archivetest/some_dir'))


    def test_cant_handle_random(self):
        assert not archive.can_handle('', __file__)
        assert not archive.can_handle('application/octet-stream', __file__)
        assert not archive.can_handle('text/x-java', __file__)
        assert not archive.can_handle('image/bitmap', __file__)
        assert not archive.can_handle('application/octet-stream', 'foo.img')
        assert not archive.can_handle('application/octet-stream', 'foo.bmp')
        assert not archive.can_handle('application/octet-stream', 'foo.jpg')

TestSupport.run(__name__)

