from __future__ import with_statement

import TestSupport
from test import TestApplication

import lockers.backend as backend
import lockers.node as node
import lockers.vcs as vcs

class BackendTests(TestSupport.TestCase):
    def setUp(self):
        TestApplication.init('/tmp/backendtest')
        self.backend = backend.get_backend()
        self.node1 = node.Node('wiki', '/Memes/Series_of_Tubes')
        self.node2 = node.Node('wiki', '/Series/Baccano!')

        if getattr(self.backend, 'backendtests_populated', False): return
        self.backend.backendtests_populated = True
        with self.backend.new_transaction() as t:
            t.save_part(self.node1, 'index', 'hi guys')
            t.commit("initial commit")
            t.save_part(self.node2, 'index', 'i like horsies')
            t.commit('second commit')

    def test_transaction_invalid(self):
        '''Make sure you can't actually use a transaction outside of a 'with'
        block.'''

        t = self.backend.new_transaction()
        self.assertRaises(RuntimeError, lambda: t.exists)
        self.failIf(t.valid())
        with t:
            self.assert_(t.exists(self.node1))
            self.assert_(t.valid())

        self.assertRaises(RuntimeError, lambda: t.exists)
        self.failIf(t.valid())

    def test_transaction_buffer(self):
        with self.backend.new_transaction() as t:
            t.save_part(self.node1, 'index', 'yo guys')
            t.commit('revise with yo')
            # commit hasn't happened yet. This could change if the transaction
            # "simulates" the operations on it
            self.assertEqual(t.get_part(self.node1, 'index'), 'hi guys')

        with self.backend.new_transaction() as t:
            self.assertEqual(t.get_part(self.node1, 'index'), 'yo guys')

        with self.backend.new_transaction() as t:
            t.save_part(self.node2, 'index', 'i like puppies')
            t.commit('revise with puppies')
            raise backend.RollbackTransaction, "i'm allergic to puppies"

        with self.backend.new_transaction() as t:
            self.assertEqual(t.get_part(self.node2, 'index'), 'i like horsies')

        with self.backend.new_transaction() as t:
            t.save_part(self.node2, 'index', 'i like kitties')
            t.commit('revise with kitties')
            t.rollback() # i can has transaction?

        # Rollback should not leave anything to be committed
        with self.backend.new_transaction() as t:
            self.assertEqual(t.get_part(self.node2, 'index'), 'i like horsies')

        with self.backend.new_transaction() as t:
            t.save_part(self.node2, 'index', 'i like pythonies')
            t.commit('revise with snakies')
            t.apply()
            raise backend.RollbackTransaction, "apply already called"

        # apply already performed the commit
        with self.backend.new_transaction() as t:
            self.assertEqual(t.get_part(self.node2, 'index'), 'i like pythonies')

        # Test that it's OK if transaction finished after an apply
        with self.backend.new_transaction() as t:
            t.save_part(self.node2, 'index', 'i like fishies')
            t.commit('revise with fishies')
            t.apply()

        with self.backend.new_transaction() as t:
            self.assertEqual(t.get_part(self.node2, 'index'), 'i like fishies')

TestSupport.run(__name__)
