import os
import sys
import TestSupport

def Fork(target=os.path.join(TestSupport.testroot, "locking_process.py"), args=("read",)):
    pid = os.fork()
    args = [str(a) for a in args]
    if pid == 0:
        os.execl(target, target, *args)
    else:
        return pid

class LockTests(TestSupport.TestCase):
    def runprocesses(self, ps):
        for p in ps: os.waitpid(p, 0)

    @classmethod
    def nesting_correct(cls, results):
        stack = []
        for c in results:
            if c == '(' or c == '[':
                stack.append(c)
            if c == ')' or c == ']':
                if c == ')':
                    if stack[-1] != '(': return False
                elif c == ']':
                    if stack[-1] != '[': return False
                else: return False
                stack.pop()
        if stack != []: return False
        return True

    @classmethod
    def maxdepth(cls, results):
        maxdepth = 0
        depth = 0
        for c in results:
            if c == '(' or c == '[':
                depth += 1
            if c == ')' or c == ']':
                if depth > maxdepth: maxdepth = depth
                depth -= 1
        return maxdepth

    def recycle(self, pidlist, f, pidlist2, maxnum=None):
        '''Recycle the processes in pidlist unlil pidlist2 is empty.

        f is a function that returns a replacement for a child in pidlist.'''
        newnum = 0
        while True:
            pid, status = os.wait()
            if pid in pidlist2:
                pidlist2.remove(pid)
                if pidlist2 == []: return True
            else:
                index = pidlist.index(pid)
                pidlist[index] = f()
                newnum += 1
                if newnum == maxnum: raise Exception, "starved while recycling"


    def get_results(self):
        results = file('results').read()
        self.assert_(self.nesting_correct(results), "improper nesting: "+results)
        return results

    def test_readers_share(self):
        ps = [Fork(args=("read",)),
              Fork(args=("read",))]
        self.runprocesses(ps)
        results = self.get_results()
        self.assertEqual(results, '(())', "readers did not nest")

    def test_writers_dont_share(self):
        ps = [Fork(args=("write",)),
              Fork(args=("write",))]
        self.runprocesses(ps)
        results = self.get_results()
        self.assertEqual(results, '[][]', "writers must not nest")

    def test_writers_dont_starve(self):
        wp = Fork(args=("write", 1))
        readers = [Fork(args=("read", 0, 2)), Fork(args=("read", 1, 2))]

        result = self.recycle(readers, lambda: Fork(args=("read", 0, 2)), [wp],
                              maxnum=10)

        self.runprocesses(readers)
        results = self.get_results()

        self.failUnless(result == True, "writer was starved")

    def test_readers_multiplex(self):
        writers = [Fork(args=("write", 0, 2)),\
                   Fork(args=("write", 1, 2))]
        readers = [Fork(args=("read", 0, 1)), \
                   Fork(args=("read", 0, 1)), \
                   Fork(args=("read", 0, 1)), \
                   Fork(args=("read", 0, 1)), \
                   Fork(args=("read", 0, 1))]

        self.recycle(writers, lambda: Fork(args=('write', 0, 2)), readers)
        self.runprocesses(writers)

        results = self.get_results()
        maxdepth = self.maxdepth(results)
        self.assert_(maxdepth > 3, "readers did not nest: "+results)

    def test_readers_interleave(self):
        writers = []
        readers = []
        writers.append(Fork(args=("write", 0, 2)))
        readers.append(Fork(args=("read", 0, 1)))
        writers.append(Fork(args=("write", 0.5, 2)))
        readers.append(Fork(args=("read", 0, 1)))
        writers.append(Fork(args=("write", 1, 2)))
        readers.append(Fork(args=("read", 0, 1)))
        writers.append(Fork(args=("write", 1.5, 2)))
        readers.append(Fork(args=("read", 0, 1)))
        writers.append(Fork(args=("write", 2, 2)))
        readers.append(Fork(args=("read", 0, 1)))

        self.recycle(writers, lambda: Fork(args=('write', 0, 2)), readers)
        self.runprocesses(writers)

        results = self.get_results()
        maxdepth = self.maxdepth(results)
        self.assert_(maxdepth >= 3, "readers did not nest: "+results)

    def setUp(self):
        # try to delete these files, just in case they're there
        try:
            os.unlink('results')
        except:
            pass

    def tearDown(self):
        try:
            os.unlink('lockfile1')
            os.unlink('lockfile2')
            os.unlink('results')
        except:
            pass    # probably an error during the test

if __name__ != '__main__':
    if TestSupport.FAST:
        LockTests = None

TestSupport.run(__name__)
