import os, os.path
import TestSupport
from TestSupport import testroot
from lockers import pagination
from lockers.request import LockersRequest
from lockers.node import Node
from lockers.pagination import Page

class PaginationTests(TestSupport.TestCase):
    def setUp(self):
        self.items = range(1, 26)  # [1..25]
        self.request = LockersRequest.blank('/foo/bar', {})
        self.node = Node(self.request)

    def test_zip(self):
        p = pagination.Paginator(self.node, self.items, 10, skip=0)
        self.assertEqual(p.number_of_pages(), 3)
        self.assertEqual(p.slice(), range(1, 11))
        self.assertEqual(p.page, 1)
        self.assertEqual(p.pages_before(), [])
        self.assertEqual(p.pages_after(), [Page(self.node, 2, 10), Page(self.node, 3, 20)])
        assert p.at_beginning()
        assert not p.at_end()
        self.assertEqual(p.last_page(), Page(self.node, 3, 20))
        def check_first_and_last(paginator):
            self.assertEqual(paginator.first_page(), Page(self.node, 1, 0))
            self.assertEqual(paginator.last_page(), Page(self.node, 3, 20))
        check_first_and_last(p)



        p = pagination.Paginator(self.node, self.items, 10, skip=1)
        self.assertEqual(p.number_of_pages(), 3)
        self.assertEqual(p.slice(), range(2, 12))
        self.assertEqual(p.page, 2)
        self.assertEqual(p.pages_before(), [Page(self.node, 1, 0)])
        self.assertEqual(p.pages_after(), [Page(self.node, 3, 20)])
        assert not p.at_beginning()
        assert not p.at_end()
        check_first_and_last(p)
        self.assertEqual(p.next_page(), Page(self.node, 3, 11))
        self.assertEqual(p.previous_page(), Page(self.node, 1, 0))

        p = pagination.Paginator(self.node, self.items, 10, skip=11)
        self.assertEqual(p.number_of_pages(), 3)
        self.assertEqual(p.slice(), range(12, 22))
        self.assertEqual(p.page, 3)
        self.assertEqual(p.pages_before(), [Page(self.node, 1, 0), Page(self.node, 2, 10)])
        self.assertEqual(p.pages_after(), [])
        assert not p.at_beginning()
        assert not p.at_end()
        check_first_and_last(p)
        self.assertEqual(p.next_page(), Page(self.node, 4, 21))
        self.assertEqual(p.previous_page(), Page(self.node, 2, 1))



TestSupport.run(__name__)

