from __future__ import with_statement

import TestSupport
import TestApplication

from lockers import tags
from lockers import query
import lockers.backend as backend
from lockers.models.models import LockersObject, Changeset, ModelFactory
from lockers.models.fields import StrField, NumField
from lockers.node import Node
import lockers.vcs as vcs
from lockers.request import LockersRequest
from lockers.session import SessionMiddleware
import dataset1
import dataset2

class QueryTests(TestSupport.TestCase):
    def setUp(self):
        TestApplication.init('/tmp/querytest')
        self.backend = backend.get_backend()
        dataset1.populate(self.backend)
        self.rev0 = self.backend.current_revision()
        QueryTests.nodes, QueryTests.objects, QueryTests.revisions = dataset2.populate(self.backend)

    def test_next_child(self):
        with self.backend.new_transaction() as t:
            pth = t.query().next_child('files/').execute().get_one()
            self.assertEqual(pth, 'files/000000015')
            n = Node(pth)
            o = dataset2.D2Object.new(t, n)
            o.age = -14
            o.save({'message': 'Creating a new object'})

        with self.backend.new_transaction() as t:
            pth = t.query().next_child('files/').execute().get_one()
            self.assertEqual(pth, 'files/000000016')
            pth = t.query().next_child('files/', revision=dataset2.revisions[0]).execute().get_one()
            self.assertEqual(pth, 'files/000000001')

            for i in range(len(dataset2.revisions)):
                pth = t.query().next_child('files/', revision=dataset2.revisions[i]).execute().get_one()
                self.assertEqual(pth, 'files/%0.9d'%(i+1))

            self.assertRaises(StopIteration, lambda: t.query().next_child('files/', revision=self.rev0).execute().get_one())


TestSupport.run(__name__)
