from __future__ import with_statement

import TestSupport
import TestApplication

from lockers import tags
import lockers.backend as backend
from lockers.models.models import Changeset
from lockers.node import Node
import lockers.vcs as vcs
from lockers.request import LockersRequest
import dataset1
from dataset1 import RandomObject

class TagTests(TestSupport.TestCase):
    node1 = dataset1.node1
    node2 = dataset1.node2
    def setUp(self):
        TestApplication.init('/tmp/backendtest')
        self.backend = backend.get_backend()
        if not hasattr(TagTests, 'data_setup'):
            TagTests.data_setup = True
            self.obj1, self.obj2 = dataset1.populate(self.backend)
            TagTests.node0 = dataset1.node0

    def test_tag1(self):
        with self.backend.new_transaction() as t:
            # FIXME: Clumsy API
            to = tags.TagObject.get(t, self.node1)
            self.assertEqual(to.index.absolute_tags, [])
            to.index.absolute_tags.append('has foo')
            to.save(Changeset({'message': 'adding "has foo" tag'}))

        with self.backend.new_transaction() as t:
            to = tags.TagObject.get(t, self.node1)
            self.assertEqual(to.index.relative_tags, ['has foo'])

        with self.backend.new_transaction() as t:
            to = tags.TagObject.get(t, self.node0)
            self.assertEqual(to.index.absolute_tags, [])
            to.index.absolute_tags.append('greetings')
            to.save(Changeset({'message': 'tag the past'}))

        with self.backend.new_transaction() as t:
            to = tags.TagObject.get(t, self.node1)
            self.assertEqual(to.index.relative_tags, ['has foo'], 'current tags changed')

            to = tags.TagObject.get(t, self.node0)
            self.assertEqual(to.index.absolute_tags, ['greetings'], 'past tags changed')

    def test_tag2(self):
        path1 = Node('taggable', 'Something')
        with self.backend.new_transaction() as t:
            obj1 = RandomObject.new(t, path1)
            obj1.key = 'value'
            obj1.save(Changeset({'message': 'first version'}))

        rev1 = self.backend.current_revision()

        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            tagobj.index.absolute_tags.append('has key')
            tagobj.save(Changeset({'message': 'adding "has key" tag'}))

        path1.revision = rev1
        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.relative_tags, [])
            tagobj.index.absolute_tags.append('poignant')
            tagobj.save(Changeset({'message': 'adding absolute tag'}))

        path1.revision = None
        with self.backend.new_transaction() as t:
            obj2 = RandomObject(t, path1)
            obj2.key = 'skeleton'
            obj2.save(Changeset({'message': 'changing type of key'}))
        rev2 = self.backend.current_revision()

        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.absolute_tags, [])
            self.assertEqual(tagobj.index.relative_tags, ['has key'])
            tagobj.index.absolute_tags.append('still has key')
            tagobj.save(Changeset({'message': 'adding "still has" tag'}))

        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.relative_tags, ['has key', 'still has key'])

        path1.revision = rev1
        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.absolute_tags, ['poignant'])
            self.assertEqual(tagobj.index.relative_tags, [])
            tagobj.index.absolute_tags.append('sad')
            tagobj.save(Changeset({'message': 'adding synonym'}))

        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.absolute_tags, ['poignant', 'sad'])
            self.assertEqual(tagobj.index.relative_tags, [])

        path1.revision = rev2
        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.relative_tags, ['has key'])
            self.assertEqual(tagobj.index.absolute_tags, [])
            tagobj.index.absolute_tags.append('margot')
            tagobj.save(Changeset({'message': 'adding in-joke'}))

        # Make sure past tags gets both relative and absolute
        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.relative_tags, ['has key'])
            self.assertEqual(tagobj.index.absolute_tags, ['margot'])

        # Make sure current tags are undisturbed
        path1.revision = None
        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.relative_tags, ['has key', 'still has key'])
            # FIXME: in the future, this could change to be ['margot']
            self.assertEqual(tagobj.index.absolute_tags, [])
            del tagobj.index.relative_tags[1]
            tagobj.save(Changeset({'message': 'removing synonym'}))

        with self.backend.new_transaction() as t:
            tagobj = tags.TagObject.get(t, path1)
            self.assertEqual(tagobj.index.relative_tags, ['has key'])
            self.assertEqual(tagobj.index.absolute_tags, [])

TestSupport.run(__name__)
